#!/usr/bin/python
#
# Copyright Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: grove
short_description: Sends a notification to a grove.io channel
description:
  - The C(grove) module sends a message for a service to a Grove.io channel.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  channel_token:
    type: str
    description:
      - Token of the channel to post to.
    required: true
  service:
    type: str
    description:
      - Name of the service (displayed as the "user" in the message).
    default: ansible
  message_content:
    type: str
    description:
      - Message content.
      - The alias O(ignore:message) has been removed in community.general 4.0.0.
    required: true
  url:
    type: str
    description:
      - Service URL for the web client.
  icon_url:
    type: str
    description:
      - Icon for the service.
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated. This should only be used on personally controlled sites using self-signed
        certificates.
    default: true
    type: bool
author: "Jonas Pfenniger (@zimbatm)"
"""

EXAMPLES = r"""
- name: Sends a notification to a grove.io channel
  community.general.grove:
    channel_token: 6Ph62VBBJOccmtTPZbubiPzdrhipZXtg
    service: my-app
    message: 'deployed {{ target }}'
"""

from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

# ==============================================================
# do_notify_grove


def do_notify_grove(module, channel_token, service, message, url=None, icon_url=None):
    my_url = f"https://grove.io/api/notice/{channel_token}/"

    my_data = dict(service=service, message=message)
    if url is not None:
        my_data["url"] = url
    if icon_url is not None:
        my_data["icon_url"] = icon_url

    data = urlencode(my_data)
    response, info = fetch_url(module, my_url, data=data)
    if info["status"] != 200:
        module.fail_json(msg=f"failed to send notification: {info['msg']}")


# ==============================================================
# main


def main():
    module = AnsibleModule(
        argument_spec=dict(
            channel_token=dict(type="str", required=True, no_log=True),
            message_content=dict(type="str", required=True),
            service=dict(type="str", default="ansible"),
            url=dict(type="str"),
            icon_url=dict(type="str"),
            validate_certs=dict(default=True, type="bool"),
        )
    )

    channel_token = module.params["channel_token"]
    service = module.params["service"]
    message = module.params["message_content"]
    url = module.params["url"]
    icon_url = module.params["icon_url"]

    do_notify_grove(module, channel_token, service, message, url, icon_url)

    # Mission complete
    module.exit_json(msg="OK")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_group_policies_info
short_description: Information module for networks _group _policies
description:
  - Get all networks _group _policies.
  - Get networks _group _policies by id.
  - Display a group policy.
  - List the group policies in a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  groupPolicyId:
    description:
      - GroupPolicyId path parameter. Group policy ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks getNetworkGroupPolicies
    description: Complete reference of the getNetworkGroupPolicies API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-group-policies
  - name: Cisco Meraki documentation for networks getNetworkGroupPolicy
    description: Complete reference of the getNetworkGroupPolicy API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-group-policy
notes:
  - SDK Method used are
    networks.Networks.get_network_group_policies,
    networks.Networks.get_network_group_policy,
  - Paths used are
    get /networks/{networkId}/groupPolicies,
    get /networks/{networkId}/groupPolicies/{groupPolicyId},
"""

EXAMPLES = r"""
- name: Get all networks _group _policies
  cisco.meraki.networks_group_policies_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
  register: result
- name: Get networks _group _policies by id
  cisco.meraki.networks_group_policies_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
    groupPolicyId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "groupPolicyId": "string",
      "scheduling": {
        "enabled": true,
        "monday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "tuesday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "wednesday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "thursday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "friday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "saturday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "sunday": {
          "active": true,
          "from": "string",
          "to": "string"
        }
      },
      "bandwidth": {
        "settings": "string",
        "bandwidthLimits": {
          "limitUp": 0,
          "limitDown": 0
        }
      },
      "firewallAndTrafficShaping": {
        "settings": "string",
        "trafficShapingRules": [
          {
            "definitions": [
              {
                "type": "string",
                "value": "string"
              }
            ],
            "perClientBandwidthLimits": {
              "settings": "string",
              "bandwidthLimits": {
                "limitUp": 0,
                "limitDown": 0
              }
            },
            "dscpTagValue": 0,
            "pcpTagValue": 0,
            "priority": "string"
          }
        ],
        "l3FirewallRules": [
          {
            "comment": "string",
            "policy": "string",
            "protocol": "string",
            "destPort": "string",
            "destCidr": "string"
          }
        ],
        "l7FirewallRules": [
          {
            "policy": "string",
            "type": "string",
            "value": "string"
          }
        ]
      },
      "contentFiltering": {
        "allowedUrlPatterns": {
          "settings": "string",
          "patterns": [
            "string"
          ]
        },
        "blockedUrlPatterns": {
          "settings": "string",
          "patterns": [
            "string"
          ]
        },
        "blockedUrlCategories": {
          "settings": "string",
          "categories": [
            "string"
          ]
        }
      },
      "splashAuthSettings": "string",
      "vlanTagging": {
        "settings": "string",
        "vlanId": "string"
      },
      "bonjourForwarding": {
        "settings": "string",
        "rules": [
          {
            "description": "string",
            "vlanId": "string",
            "services": [
              "string"
            ]
          }
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_dedicated_host
version_added: 9.3.0
short_description: Create, update or delete (release) EC2 dedicated host
description:
  - Create, update or delete (release) EC2 dedicated host.
author:
  - "Aubin Bikouo (@abikouo)"
options:
  state:
    description:
      - Create or delete dedicated host.
    required: false
    choices: [ present, absent ]
    default: 'present'
    type: str
  host_id:
    description:
      - The IDs of the Dedicated Hosts to release or modify
    type: str
    required: false
  lookup:
    description:
      - Look up EC2 dedicated host by either O(tags) or by O(host_id).
      - If O(lookup=tag) and O(tags) is not specified then no lookup for an
        existing dedicated host is performed and a new dedicated host will be created.
      - When using O(lookup=tag), multiple matches being found will result in
        a failure and no changes will be made.
      - To change the tags of a dedicated host, use O(lookup=host_id).
    default: tag
    choices: [ 'tag', 'host_id' ]
    type: str
  instance_family:
    description:
      - Specifies the instance family to be supported by the Dedicated Hosts.
      - If you specify an instance family, the Dedicated Hosts support multiple instance types within that instance family.
      - At least one of O(instance_family) or O(instance_type) must be specified when allocating dedicated host.
      - Mutually exclusive with paramter O(instance_type).
    type: str
  instance_type:
    description:
      - Specifies the instance type to be supported by the Dedicated Hosts.
      - If you specify an instance type, the Dedicated Hosts support instances of the specified instance type only.
      - At least one of O(instance_family) or O(instance_type) must be specified when allocating dedicated host.
      - Mutually exclusive with paramter O(instance_family).
    type: str
  host_recovery:
    description:
      - Indicates whether to enable or disable host recovery for the Dedicated Host.
      - Host recovery is disabled by default.
    type: str
    default: 'off'
    choices: ['on', 'off']
  outpost_arn:
    description:
      - The Amazon Resource Name (ARN) of the Amazon Web Services Outpost on which to allocate the Dedicated Host.
    type: str
  host_maintenance:
    description:
      - Indicates whether to enable or disable host maintenance for the Dedicated Host.
    type: str
    default: 'off'
    choices: ['on', 'off']
  asset_ids:
    description:
      - The IDs of the Outpost hardware assets on which to allocate the Dedicated Hosts.
      - Targeting specific hardware assets on an Outpost can help to minimize latency between your workloads.
      - This parameter is supported only if you specify O(outpost_arn).
      - If you specify this parameter, you can omit O(quantity).
      - If you specify both O(asset_ids) and O(quantity), then the value for O(quantity) must be equal to the number of O(asset_ids) specified.
    type: list
    elements: str
  auto_placement:
    description:
      - Indicates whether the host accepts any untargeted instance launches that match its instance type configuration,
        or if it only accepts Host tenancy instance launches that specify its unique host ID.
    type: str
    default: 'off'
    choices: ['on', 'off']
  client_token:
    description:
      - case-sensitive identifier that you provide to ensure the idempotency of the request.
    type: str
  quantity:
    description:
      - The number of Dedicated Hosts to allocate to your account with these parameters.
      - Required when O(state=present).
      - If you are allocating the Dedicated Hosts on an Outpost, and you specify O(asset_ids), you can omit this parameter.
      - If you specify both O(asset_ids) and O(quantity), then the value that you specify for O(quantity) must be equal to the number of O(asset_ids) specified.
    type: int
  availability_zone:
    description:
      - The Availability Zone in which to allocate the Dedicated Host.
      - Required if O(state=present).
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
  - amazon.aws.tags
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Release dedicated hosts
- name: Release dedicated host
  amazon.aws.ec2_dedicated_host:
    host_id: "h-0123456789abcdef0"

# Allocate dedicated host using instance type
- name: Allocate dedicated host using instance_type
  amazon.aws.ec2_dedicated_host:
    availability_zone: us-east-1a
    quantity: 1
    instance_type: mac2.metal
    tags:
      Scope: Mac
"""

RETURN = r"""
host:
  description: Information about the dedicated host created/updated.
  returned: When O(state=present)
  type: complex
  contains:
    allocation_time:
      description: The time that the Dedicated Host was allocated.
      returned: always
      type: str
      sample: "2025-02-12T12:09:22+00:00"
    allows_multiple_instance_types:
      description: Indicates whether the Dedicated Host supports multiple instance types of the same instance family.
      returned: always
      type: str
      sample: "off"
    auto_placement:
      description: Whether auto-placement is on or off.
      returned: always
      type: str
      sample: "off"
    availability_zone:
      description: The Availability Zone of the Dedicated Host.
      returned: always
      type: str
      sample: "us-east-1b"
    availability_zone_id:
      description: The ID of the Availability Zone in which the Dedicated Host is allocated.
      returned: always
      type: str
      sample: "use1-az6"
    available_capacity:
      description: Information about the instances running on the Dedicated Host.
      returned: always
      type: dict
      contains:
        available_v_cpus:
          description: The number of vCPUs available for launching instances onto the Dedicated Host.
          returned: always
          type: int
          sample: 8
        available_instance_capacity:
          description: The number of instances that can be launched onto the Dedicated Host.
          returned: always
          type: complex
          contains:
            available_capacity:
              description: The number of instances that can be launched onto the Dedicated Host.
              returned: always
              type: int
              sample: 1
            instance_type:
              description: The instance type supported by the Dedicated Host.
              returned: always
              type: str
              sample: "mac2.metal"
            total_capacity:
              description: The total number of instances that can be launched onto the Dedicated Host if there are no instances running on it.
              returned: always
              type: dict
              sample: 1
    host_id:
      description: The ID of the Dedicated Host.
      returned: always
      type: str
      sample: "h-03f51341e6e39f848"
    client_token:
      description: Unique, case-sensitive identifier that you provide to ensure the idempotency of the request.
      returned: always
      type: str
      sample: "token-0123456789a"
    host_properties:
      description: The hardware specifications of the Dedicated Host.
      returned: always
      type: dict
      contains:
        cores:
          description: The number of cores on the Dedicated Host.
          returned: always
          type: int
          sample: 8
        instance_type:
          description: The instance type supported by the Dedicated Host.
          returned: always
          type: str
          sample: "mac2.metal"
        instance_family:
          description: The instance family supported by the Dedicated Host.
          returned: if defined
          type: str
          sample: "mac2"
        sockets:
          description: The number of sockets on the Dedicated Host.
          returned: always
          type: int
          sample: 1
        total_v_cpus:
          description: The total number of vCPUs on the Dedicated Host.
          returned: always
          type: int
          sample: 8
    host_reservation_id:
      description: The reservation ID of the Dedicated Host.
      returned: always
      type: str
    instances:
      description: The IDs and instance type that are currently running on the Dedicated Host.
      returned: always
      type: complex
      contains:
        instance_id:
          description: The ID of instance that is running on the Dedicated Host.
          returned: always
          type: str
          sample: "i-0123456789abcd"
        instance_type:
          description: The instance type of the running instance.
          returned: always
          type: str
          sample: "ec2.micro"
        owner_id:
          description: The ID of the Amazon Web Services account that owns the instance.
          returned: always
          type: str
          sample: "0123456789"
    state:
      description: The state of the Dedicated Host.
      returned: always
      type: str
      sample: "available"
    release_time:
      description: The time that the Dedicated Host was released.
      returned: always
      type: str
      sample: "2025-02-12T12:09:22+00:00"
    host_recovery:
      description: Indicates whether host recovery is enabled or disabled for the Dedicated Host.
      returned: always
      type: str
      sample: "off"
    owner_id:
      description: The ID of the Amazon Web Services account that owns the Dedicated Host.
      returned: always
      type: str
      sample: "0123456789"
    member_of_service_linked_resource_group:
      description: Indicates whether the Dedicated Host is in a host resource group.
      returned: always
      type: bool
      sample: false
    outpost_arn:
      description: The Amazon Resource Name (ARN) of the Amazon Web Services Outpost on which the Dedicated Host is allocated.
      returned: always
      type: str
      sample: "arn:aws:outposts:us-east-1:0123012301230123:outpost/op-0123456789abcdef0"
    host_maintenance:
      description: Indicates whether host maintenance is enabled or disabled for the Dedicated Host.
      returned: always
      type: str
      sample: "off"
    asset_id:
      description: The ID of the Outpost hardware asset on which the Dedicated Host is allocated.
      returned: always
      type: str
      sample: "abcdefgh"
    tags:
      description: Dictionary of tags added to the dedicated host.
      returned: always
      type: dict
      sample: {}
"""

from typing import Any
from typing import Dict
from typing import List
from typing import NoReturn
from typing import Optional

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import allocate_ec2_dedicated_hosts
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_ec2_dedicated_hosts
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import modify_ec2_dedicated_hosts
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import release_ec2_dedicated_host
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


def describe_host(client, host_id: str) -> Optional[Dict[str, Any]]:
    host_info = None
    hosts = describe_ec2_dedicated_hosts(client, HostIds=[host_id])
    if hosts:
        host_info = hosts[0]
    return host_info


def tags_match(match_tags: Dict[str, str], candidate_tags: Dict[str, str]) -> bool:
    return all((k in candidate_tags and candidate_tags[k] == v for k, v in match_tags.items()))


def get_ec2_dedicated_host(client, module: AnsibleAWSModule) -> Optional[Dict[str, Any]]:
    host_info = None
    # Select all states except 'released' and 'released-permanent-failure'
    params = {"Filters": [{"Name": "state", "Values": ["available", "under-assessment", "permanent-failure"]}]}

    lookup = module.params.get("lookup")
    host_id = module.params.get("host_id")
    ansible_tags = module.params.get("tags")

    if lookup == "host_id":
        params["HostIds"] = [host_id]
    elif lookup == "tag" and not ansible_tags:
        # lookup == 'tag' but tags was not provided
        return host_info
    hosts = describe_ec2_dedicated_hosts(client, **params)
    if hosts:
        if lookup == "tag":
            count = 0
            for h in hosts:
                aws_tags = boto3_tag_list_to_ansible_dict(h.get("Tags", {}))
                if aws_tags == ansible_tags:
                    host_info = h
                    count += 1
            if count > 1:
                module.fail_json(msg=f"Tags provided do not identify a unique dedicated host ({count} found).")
        else:
            host_info = hosts[0]

    return host_info


def format_output(host: Dict[str, Any]) -> List[Dict[str, Any]]:
    host["tags"] = boto3_tag_list_to_ansible_dict(host.pop("Tags", {}))
    return camel_dict_to_snake_dict(host, ignore_list=["tags"])


def release_host(client, module: AnsibleAWSModule, existing: Dict[str, Any]) -> NoReturn:
    if not existing:
        module.exit_json(changed=False)
    host_id = module.params.get("host_id")
    if module.check_mode:
        module.exit_json(changed=True, msg=f"Would have released dedicated host '{host_id}' if not in check mode.")
    changed = release_ec2_dedicated_host(client, host_id)
    module.exit_json(changed=changed, host=format_output(existing))


def create_or_update_host(client, module: AnsibleAWSModule, existing: Dict[str, Any]) -> NoReturn:
    host_id = module.params.get("host_id")
    changed = False
    if existing:
        # Update dedicated host
        # Check if one of the following has changed ('HostRecovery', 'InstanceType', 'InstanceFamily', 'HostMaintenance', 'AutoPlacement')
        host_maintenance = module.params.get("host_maintenance")
        instance_type = module.params.get("instance_type")
        instance_family = module.params.get("instance_family")
        auto_placement = module.params.get("auto_placement")
        host_recovery = module.params.get("host_recovery")
        params_to_update = {}
        if host_recovery != existing.get("HostRecovery"):
            params_to_update["HostRecovery"] = host_recovery
        if instance_type != existing.get("HostProperties", {}).get("InstanceType"):
            params_to_update["InstanceType"] = instance_type
        if instance_family != existing.get("HostProperties", {}).get("InstanceFamily"):
            params_to_update["InstanceFamily"] = instance_family
        if host_maintenance != existing.get("HostMaintenance"):
            params_to_update["HostMaintenance"] = host_maintenance
        if auto_placement != existing.get("AutoPlacement"):
            params_to_update["AutoPlacement"] = auto_placement

        if params_to_update:
            if module.check_mode:
                module.exit_json(changed=True, msg="Would have update dedicated host if not in check mode.")

            result = modify_ec2_dedicated_hosts(client, host_id=host_id, **params_to_update)
            if result.get("Unsuccessful"):
                code = result["Unsuccessful"][0]["Error"]["Code"]
                message = result["Unsuccessful"][0]["Error"]["Message"]
                module.fail_json(
                    msg=f"The Dedicated Hosts '{host_id}' could not be modified. Code='{code}' Message = '{message}'"
                )
            changed = True
    else:
        # Allocate dedicated host
        if module.check_mode:
            module.exit_json(changed=True, msg="Would have allocate dedicated host if not in check mode.")

        availability_zone = module.params.get("availability_zone")
        instance_family = module.params.get("instance_family")
        host_recovery = module.params.get("host_recovery")
        outpost_arn = module.params.get("outpost_arn")
        host_maintenance = module.params.get("host_maintenance")
        auto_placement = module.params.get("auto_placement")
        asset_ids = module.params.get("asset_ids")
        client_token = module.params.get("client_token")
        instance_type = module.params.get("instance_type")
        quantity = module.params.get("quantity")

        params = {}
        if instance_family:
            params["InstanceFamily"] = instance_family
        if host_recovery:
            params["HostRecovery"] = host_recovery
        if outpost_arn:
            params["OutpostArn"] = outpost_arn
        if host_maintenance:
            params["HostMaintenance"] = host_maintenance
        if asset_ids:
            params["AssetIds"] = asset_ids
        if client_token:
            params["ClientToken"] = client_token
        if instance_type:
            params["InstanceType"] = instance_type
        if quantity:
            params["Quantity"] = quantity

        host_id = allocate_ec2_dedicated_hosts(client, availability_zone=availability_zone, **params)[0]
        changed = True

    # Ensure tags
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")
    changed |= ensure_ec2_tags(client, module, host_id, tags=tags, purge_tags=purge_tags)

    host_info = describe_host(client, host_id)
    module.exit_json(changed=changed, host=format_output(host_info))


def main():
    module = AnsibleAWSModule(
        argument_spec=dict(
            state=dict(type="str", choices=["present", "absent"], default="present"),
            availability_zone=dict(type="str", required=False),
            host_id=dict(),
            lookup=dict(type="str", choices=["tag", "host_id"], default="tag"),
            instance_family=dict(type="str"),
            host_recovery=dict(type="str", choices=["on", "off"], default="off"),
            outpost_arn=dict(),
            host_maintenance=dict(type="str", choices=["on", "off"], default="off"),
            auto_placement=dict(type="str", choices=["on", "off"], default="off"),
            asset_ids=dict(type="list", elements="str"),
            client_token=dict(no_log=False),
            instance_type=dict(),
            quantity=dict(type="int"),
            tags=dict(type="dict", aliases=["resource_tags"]),
            purge_tags=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
        mutually_exclusive=[["instance_family", "instance_type"]],
        required_if=[
            ["state", "present", ["availability_zone", "quantity"]],
            ["state", "absent", ["host_id"]],
            ["lookup", "host_id", ["host_id"]],
        ],
    )

    client = module.client("ec2")
    state = module.params.get("state")

    existing = get_ec2_dedicated_host(client, module)

    try:
        if state == "absent":
            # Release EC2 dedicated host
            release_host(client, module, existing)
        else:
            create_or_update_host(client, module, existing)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)


if __name__ == "__main__":
    main()



# Copyright (c) 2021, Andrew Pantuso (@ajpantuso) <ajpantuso@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import typing as t
from base64 import b64decode, b64encode
from getpass import getuser
from socket import gethostname

try:
    from cryptography import __version__ as CRYPTOGRAPHY_VERSION
    from cryptography.exceptions import InvalidSignature, UnsupportedAlgorithm
    from cryptography.hazmat.primitives import hashes, serialization
    from cryptography.hazmat.primitives.asymmetric import dsa, ec, padding, rsa
    from cryptography.hazmat.primitives.asymmetric.ed448 import Ed448PrivateKey
    from cryptography.hazmat.primitives.asymmetric.ed25519 import (
        Ed25519PrivateKey,
        Ed25519PublicKey,
    )

    HAS_OPENSSH_SUPPORT = True

    _ALGORITHM_PARAMETERS = {
        "rsa": {
            "default_size": 2048,
            "valid_sizes": range(1024, 16384),
            "signer_params": {
                "padding": padding.PSS(
                    mgf=padding.MGF1(hashes.SHA256()),
                    salt_length=padding.PSS.MAX_LENGTH,
                ),
                "algorithm": hashes.SHA256(),
            },
        },
        "dsa": {
            "default_size": 1024,
            "valid_sizes": [1024],
            "signer_params": {
                "algorithm": hashes.SHA256(),
            },
        },
        "ed25519": {
            "default_size": 256,
            "valid_sizes": [256],
            "signer_params": {},
        },
        "ecdsa": {
            "default_size": 256,
            "valid_sizes": [256, 384, 521],
            "signer_params": {
                "signature_algorithm": ec.ECDSA(hashes.SHA256()),
            },
            "curves": {
                256: ec.SECP256R1(),
                384: ec.SECP384R1(),
                521: ec.SECP521R1(),
            },
        },
    }
except ImportError:
    HAS_OPENSSH_SUPPORT = False
    CRYPTOGRAPHY_VERSION = "0.0"
    _ALGORITHM_PARAMETERS = {}

try:
    from cryptography.hazmat.primitives.asymmetric.mldsa import (
        MLDSA44PrivateKey,
        MLDSA65PrivateKey,
        MLDSA87PrivateKey,
    )

    HAS_MLDSA_SUPPORT = True
except ImportError:  # pragma: no cover

    class MLDSA44PrivateKey:  # type: ignore[no-redef]
        pass

    class MLDSA65PrivateKey:  # type: ignore[no-redef]
        pass

    class MLDSA87PrivateKey:  # type: ignore[no-redef]
        pass

    HAS_MLDSA_SUPPORT = False

from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    is_potential_certificate_issuer_private_key,
)

if t.TYPE_CHECKING:  # pragma: no cover
    KeyFormat = t.Literal["SSH", "PKCS8", "PKCS1"]  # pragma: no cover
    KeySerializationFormat = t.Literal["PEM", "DER", "SSH"]  # pragma: no cover
    KeyType = t.Literal["rsa", "dsa", "ed25519", "ecdsa"]  # pragma: no cover
    PubKeyType = t.Literal[
        "rsa", "dsa", "ed25519", "ecdsa-nistp256", "ecdsa-nistp384", "ecdsa-nistp521"
    ]  # pragma: no cover

    PrivateKeyTypes = t.Union[  # noqa: UP007
        rsa.RSAPrivateKey,
        dsa.DSAPrivateKey,
        ec.EllipticCurvePrivateKey,
        Ed25519PrivateKey,
    ]  # pragma: no cover
    PublicKeyTypes = t.Union[  # noqa: UP007
        rsa.RSAPublicKey, dsa.DSAPublicKey, ec.EllipticCurvePublicKey, Ed25519PublicKey
    ]  # pragma: no cover

    from cryptography.hazmat.primitives.asymmetric.types import (
        PublicKeyTypes as AllPublicKeyTypes,  # pragma: no cover
    )


_TEXT_ENCODING = "UTF-8"


class OpenSSHError(Exception):
    pass


class InvalidAlgorithmError(OpenSSHError):
    pass


class InvalidCommentError(OpenSSHError):
    pass


class InvalidDataError(OpenSSHError):
    pass


class InvalidPrivateKeyFileError(OpenSSHError):
    pass


class InvalidPublicKeyFileError(OpenSSHError):
    pass


class InvalidKeyFormatError(OpenSSHError):
    pass


class InvalidKeySizeError(OpenSSHError):
    pass


class InvalidKeyTypeError(OpenSSHError):
    pass


class InvalidPassphraseError(OpenSSHError):
    pass


class InvalidSignatureError(OpenSSHError):
    pass


_AsymmetricKeypair = t.TypeVar("_AsymmetricKeypair", bound="AsymmetricKeypair")


class AsymmetricKeypair:
    """Container for newly generated asymmetric key pairs or those loaded from existing files"""

    @classmethod
    def generate(
        cls: type[_AsymmetricKeypair],
        *,
        keytype: KeyType = "rsa",
        size: int | None = None,
        passphrase: bytes | None = None,
    ) -> _AsymmetricKeypair:
        """Returns an Asymmetric_Keypair object generated with the supplied parameters
        or defaults to an unencrypted RSA-2048 key

        :keytype: One of rsa, dsa, ecdsa, ed25519
        :size: The key length for newly generated keys
        :passphrase: Secret of type Bytes used to encrypt the private key being generated
        """

        if keytype not in _ALGORITHM_PARAMETERS:
            raise InvalidKeyTypeError(
                f"{keytype} is not a valid keytype. Valid keytypes are {', '.join(_ALGORITHM_PARAMETERS)}"
            )

        if not size:
            size = _ALGORITHM_PARAMETERS[keytype]["default_size"]  # type: ignore
        else:
            if size not in _ALGORITHM_PARAMETERS[keytype]["valid_sizes"]:  # type: ignore
                raise InvalidKeySizeError(
                    f"{size} is not a valid key size for {keytype} keys"
                )
        size = t.cast(int, size)

        privatekey: PrivateKeyTypes
        if passphrase:
            encryption_algorithm = get_encryption_algorithm(passphrase)
        else:
            encryption_algorithm = serialization.NoEncryption()

        if keytype == "rsa":
            privatekey = rsa.generate_private_key(
                # Public exponent should always be 65537 to prevent issues
                # if improper padding is used during signing
                public_exponent=65537,
                key_size=size,
            )
        elif keytype == "dsa":
            privatekey = dsa.generate_private_key(
                key_size=size,
            )
        elif keytype == "ed25519":
            privatekey = Ed25519PrivateKey.generate()
        elif keytype == "ecdsa":
            privatekey = ec.generate_private_key(
                _ALGORITHM_PARAMETERS["ecdsa"]["curves"][size],  # type: ignore
            )

        publickey = privatekey.public_key()

        return cls(
            keytype=keytype,
            size=size,
            privatekey=privatekey,
            publickey=publickey,
            encryption_algorithm=encryption_algorithm,
        )

    @classmethod
    def load(
        cls: type[_AsymmetricKeypair],
        *,
        path: str | os.PathLike,
        passphrase: bytes | None = None,
        private_key_format: KeySerializationFormat = "PEM",
        public_key_format: KeySerializationFormat = "PEM",
        no_public_key: bool = False,
    ) -> _AsymmetricKeypair:
        """Returns an Asymmetric_Keypair object loaded from the supplied file path

        :path: A path to an existing private key to be loaded
        :passphrase: Secret of type bytes used to decrypt the private key being loaded
        :private_key_format: Format of private key to be loaded
        :public_key_format: Format of public key to be loaded
        :no_public_key: Set 'True' to only load a private key and automatically populate the matching public key
        """

        if passphrase:
            encryption_algorithm = get_encryption_algorithm(passphrase)
        else:
            encryption_algorithm = serialization.NoEncryption()

        privatekey = load_privatekey(
            path=path, passphrase=passphrase, key_format=private_key_format
        )
        publickey: AllPublicKeyTypes
        if no_public_key:
            publickey = privatekey.public_key()
        else:
            # TODO: Maybe we should check whether the public key actually fits the private key?
            publickey = load_publickey(
                path=str(path) + ".pub", key_format=public_key_format
            )

        # Ed25519 keys are always of size 256 and do not have a key_size attribute
        if isinstance(privatekey, Ed25519PrivateKey):
            size: int = _ALGORITHM_PARAMETERS["ed25519"]["default_size"]  # type: ignore
        else:
            size = privatekey.key_size

        keytype: KeyType
        if isinstance(privatekey, rsa.RSAPrivateKey):
            keytype = "rsa"
            if not isinstance(publickey, rsa.RSAPublicKey):
                raise InvalidKeyTypeError(
                    f"Private key is an RSA key, but public key is of type '{type(publickey)}'"
                )
        elif isinstance(privatekey, dsa.DSAPrivateKey):
            keytype = "dsa"
            if not isinstance(publickey, dsa.DSAPublicKey):
                raise InvalidKeyTypeError(
                    f"Private key is a DSA key, but public key is of type '{type(publickey)}'"
                )
        elif isinstance(privatekey, ec.EllipticCurvePrivateKey):
            keytype = "ecdsa"
            if not isinstance(publickey, ec.EllipticCurvePublicKey):
                raise InvalidKeyTypeError(
                    f"Private key is an Elliptic Curve key, but public key is of type '{type(publickey)}'"
                )
        elif isinstance(privatekey, Ed25519PrivateKey):
            keytype = "ed25519"
            if not isinstance(publickey, Ed25519PublicKey):
                raise InvalidKeyTypeError(
                    f"Private key is an Ed25519 key, but public key is of type '{type(publickey)}'"
                )
        else:
            raise InvalidKeyTypeError(f"Key type '{type(privatekey)}' is not supported")

        return cls(
            keytype=keytype,
            size=size,
            privatekey=privatekey,
            publickey=publickey,
            encryption_algorithm=encryption_algorithm,
        )

    def __init__(
        self,
        *,
        keytype: KeyType,
        size: int,
        privatekey: PrivateKeyTypes,
        publickey: PublicKeyTypes,
        encryption_algorithm: serialization.KeySerializationEncryption,
    ) -> None:
        """
        :keytype: One of rsa, dsa, ecdsa, ed25519
        :size: The key length for the private key of this key pair
        :privatekey: Private key object of this key pair
        :publickey: Public key object of this key pair
        :encryption_algorithm: Hashed secret used to encrypt the private key of this key pair
        """

        self.__size = size
        self.__keytype = keytype
        self.__privatekey = privatekey
        self.__publickey = publickey
        self.__encryption_algorithm = encryption_algorithm

        try:
            self.verify(signature=self.sign(b"message"), data=b"message")
        except InvalidSignatureError as e:
            raise InvalidPublicKeyFileError(
                "The private key and public key of this keypair do not match"
            ) from e

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, AsymmetricKeypair):
            return NotImplemented

        return compare_publickeys(
            self.public_key, other.public_key
        ) and compare_encryption_algorithms(
            self.encryption_algorithm, other.encryption_algorithm
        )

    def __ne__(self, other: object) -> bool:
        return not self == other

    @property
    def private_key(self) -> PrivateKeyTypes:
        """Returns the private key of this key pair"""

        return self.__privatekey

    @property
    def public_key(self) -> PublicKeyTypes:
        """Returns the public key of this key pair"""

        return self.__publickey

    @property
    def size(self) -> int:
        """Returns the size of the private key of this key pair"""

        return self.__size

    @property
    def key_type(self) -> KeyType:
        """Returns the key type of this key pair"""

        return self.__keytype

    @property
    def encryption_algorithm(self) -> serialization.KeySerializationEncryption:
        """Returns the key encryption algorithm of this key pair"""

        return self.__encryption_algorithm

    def sign(self, data: bytes) -> bytes:
        """Returns signature of data signed with the private key of this key pair

        :data: byteslike data to sign
        """

        try:
            return self.__privatekey.sign(
                data,
                **_ALGORITHM_PARAMETERS[self.__keytype]["signer_params"],  # type: ignore
            )
        except TypeError as e:
            raise InvalidDataError(e) from e

    def verify(self, *, signature: bytes, data: bytes) -> None:
        """Verifies that the signature associated with the provided data was signed
        by the private key of this key pair.

        :signature: signature to verify
        :data: byteslike data signed by the provided signature
        """
        try:
            self.__publickey.verify(
                signature,
                data,
                **_ALGORITHM_PARAMETERS[self.__keytype]["signer_params"],  # type: ignore
            )
        except InvalidSignature as e:
            raise InvalidSignatureError from e

    def update_passphrase(self, passphrase: bytes | None = None) -> None:
        """Updates the encryption algorithm of this key pair

        :passphrase: Byte secret used to encrypt this key pair
        """

        if passphrase:
            self.__encryption_algorithm = get_encryption_algorithm(passphrase)
        else:
            self.__encryption_algorithm = serialization.NoEncryption()


_OpensshKeypair = t.TypeVar("_OpensshKeypair", bound="OpensshKeypair")


class OpensshKeypair:
    """Container for OpenSSH encoded asymmetric key pairs"""

    @classmethod
    def generate(
        cls: type[_OpensshKeypair],
        *,
        keytype: KeyType = "rsa",
        size: int | None = None,
        passphrase: bytes | None = None,
        comment: str | None = None,
    ) -> _OpensshKeypair:
        """Returns an Openssh_Keypair object generated using the supplied parameters or defaults to a RSA-2048 key

        :keytype: One of rsa, dsa, ecdsa, ed25519
        :size: The key length for newly generated keys
        :passphrase: Secret of type Bytes used to encrypt the newly generated private key
        :comment: Comment for a newly generated OpenSSH public key
        """

        if comment is None:
            comment = f"{getuser()}@{gethostname()}"

        asym_keypair = AsymmetricKeypair.generate(
            keytype=keytype, size=size, passphrase=passphrase
        )
        openssh_privatekey = cls.encode_openssh_privatekey(
            asym_keypair=asym_keypair, key_format="SSH"
        )
        openssh_publickey = cls.encode_openssh_publickey(
            asym_keypair=asym_keypair, comment=comment
        )
        fingerprint = calculate_fingerprint(openssh_publickey)

        return cls(
            asym_keypair=asym_keypair,
            openssh_privatekey=openssh_privatekey,
            openssh_publickey=openssh_publickey,
            fingerprint=fingerprint,
            comment=comment,
        )

    @classmethod
    def load(
        cls: type[_OpensshKeypair],
        *,
        path: str | os.PathLike,
        passphrase: bytes | None = None,
        no_public_key: bool = False,
    ) -> _OpensshKeypair:
        """Returns an Openssh_Keypair object loaded from the supplied file path

        :path: A path to an existing private key to be loaded
        :passphrase: Secret used to decrypt the private key being loaded
        :no_public_key: Set 'True' to only load a private key and automatically populate the matching public key
        """

        if no_public_key:
            comment = ""
        else:
            comment = extract_comment(str(path) + ".pub")

        asym_keypair = AsymmetricKeypair.load(
            path=path,
            passphrase=passphrase,
            private_key_format="SSH",
            public_key_format="SSH",
            no_public_key=no_public_key,
        )
        openssh_privatekey = cls.encode_openssh_privatekey(
            asym_keypair=asym_keypair, key_format="SSH"
        )
        openssh_publickey = cls.encode_openssh_publickey(
            asym_keypair=asym_keypair, comment=comment
        )
        fingerprint = calculate_fingerprint(openssh_publickey)

        return cls(
            asym_keypair=asym_keypair,
            openssh_privatekey=openssh_privatekey,
            openssh_publickey=openssh_publickey,
            fingerprint=fingerprint,
            comment=comment,
        )

    @staticmethod
    def encode_openssh_privatekey(
        *, asym_keypair: AsymmetricKeypair, key_format: KeyFormat
    ) -> bytes:
        """Returns an OpenSSH encoded private key for a given keypair

        :asym_keypair: Asymmetric_Keypair from the private key is extracted
        :key_format: Format of the encoded private key.
        """

        if key_format == "SSH":
            privatekey_format = serialization.PrivateFormat.OpenSSH
        elif key_format == "PKCS8":
            privatekey_format = serialization.PrivateFormat.PKCS8
        elif key_format == "PKCS1":
            if asym_keypair.key_type == "ed25519":
                raise InvalidKeyFormatError(
                    "ed25519 keys cannot be represented in PKCS1 format"
                )
            privatekey_format = serialization.PrivateFormat.TraditionalOpenSSL
        else:
            raise InvalidKeyFormatError(
                "The accepted private key formats are SSH, PKCS8, and PKCS1"
            )

        encoded_privatekey = asym_keypair.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=privatekey_format,
            encryption_algorithm=asym_keypair.encryption_algorithm,
        )

        return encoded_privatekey

    @staticmethod
    def encode_openssh_publickey(
        *, asym_keypair: AsymmetricKeypair, comment: str
    ) -> bytes:
        """Returns an OpenSSH encoded public key for a given keypair

        :asym_keypair: Asymmetric_Keypair from the public key is extracted
        :comment: Comment to apply to the end of the returned OpenSSH encoded public key
        """
        encoded_publickey = asym_keypair.public_key.public_bytes(
            encoding=serialization.Encoding.OpenSSH,
            format=serialization.PublicFormat.OpenSSH,
        )

        validate_comment(comment)

        encoded_publickey += (
            (b" " + comment.encode(encoding=_TEXT_ENCODING)) if comment else b""
        )

        return encoded_publickey

    def __init__(
        self,
        *,
        asym_keypair: AsymmetricKeypair,
        openssh_privatekey: bytes,
        openssh_publickey: bytes,
        fingerprint: str,
        comment: str | None,
    ) -> None:
        """
        :asym_keypair: An Asymmetric_Keypair object from which the OpenSSH encoded keypair is derived
        :openssh_privatekey: An OpenSSH encoded private key
        :openssh_privatekey: An OpenSSH encoded public key
        :fingerprint: The fingerprint of the OpenSSH encoded public key of this keypair
        :comment: Comment applied to the OpenSSH public key of this keypair
        """

        self.__asym_keypair = asym_keypair
        self.__openssh_privatekey = openssh_privatekey
        self.__openssh_publickey = openssh_publickey
        self.__fingerprint = fingerprint
        self.__comment = comment

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, OpensshKeypair):
            return NotImplemented

        return (
            self.asymmetric_keypair == other.asymmetric_keypair
            and self.comment == other.comment
        )

    @property
    def asymmetric_keypair(self) -> AsymmetricKeypair:
        """Returns the underlying asymmetric key pair of this OpenSSH encoded key pair"""

        return self.__asym_keypair

    @property
    def private_key(self) -> bytes:
        """Returns the OpenSSH formatted private key of this key pair"""

        return self.__openssh_privatekey

    @property
    def public_key(self) -> bytes:
        """Returns the OpenSSH formatted public key of this key pair"""

        return self.__openssh_publickey

    @property
    def size(self) -> int:
        """Returns the size of the private key of this key pair"""

        return self.__asym_keypair.size

    @property
    def key_type(self) -> KeyType:
        """Returns the key type of this key pair"""

        return self.__asym_keypair.key_type

    @property
    def fingerprint(self) -> str:
        """Returns the fingerprint (SHA256 Hash) of the public key of this key pair"""

        return self.__fingerprint

    @property
    def comment(self) -> str | None:
        """Returns the comment applied to the OpenSSH formatted public key of this key pair"""

        return self.__comment

    @comment.setter
    def comment(self, comment: str) -> bytes:
        """Updates the comment applied to the OpenSSH formatted public key of this key pair

        :comment: Text to update the OpenSSH public key comment
        """

        validate_comment(comment)

        self.__comment = comment
        encoded_comment = (
            f" {self.__comment}".encode(encoding=_TEXT_ENCODING)
            if self.__comment
            else b""
        )
        self.__openssh_publickey = (
            b" ".join(self.__openssh_publickey.split(b" ", 2)[:2]) + encoded_comment
        )
        return self.__openssh_publickey

    def update_passphrase(self, passphrase: bytes | None) -> None:
        """Updates the passphrase used to encrypt the private key of this keypair

        :passphrase: Text secret used for encryption
        """

        self.__asym_keypair.update_passphrase(passphrase)
        self.__openssh_privatekey = OpensshKeypair.encode_openssh_privatekey(
            asym_keypair=self.__asym_keypair, key_format="SSH"
        )


def load_privatekey(
    *,
    path: str | os.PathLike,
    passphrase: bytes | None,
    key_format: KeySerializationFormat,
) -> PrivateKeyTypes:
    privatekey_loaders = {
        "PEM": serialization.load_pem_private_key,
        "DER": serialization.load_der_private_key,
        "SSH": serialization.load_ssh_private_key,
    }

    try:
        privatekey_loader = privatekey_loaders[key_format]
    except KeyError as e:
        raise InvalidKeyFormatError(
            f"{key_format} is not a valid key format ({','.join(privatekey_loaders)})"
        ) from e

    if not os.path.exists(path):
        raise InvalidPrivateKeyFileError(f"No file was found at {path}")

    try:
        with open(path, "rb") as f:
            content = f.read()

        try:
            privatekey = privatekey_loader(
                data=content,
                password=passphrase,
            )
        except ValueError as exc:
            # Revert to PEM if key could not be loaded in SSH format
            if key_format == "SSH":
                privatekey = privatekey_loaders["PEM"](
                    data=content,
                    password=passphrase,
                )
            else:
                raise InvalidPrivateKeyFileError(exc) from exc
    except ValueError as e:
        raise InvalidPrivateKeyFileError(e) from e
    except TypeError as e:
        raise InvalidPassphraseError(e) from e
    except UnsupportedAlgorithm as e:
        raise InvalidAlgorithmError(e) from e

    if not is_potential_certificate_issuer_private_key(privatekey) or isinstance(
        privatekey, Ed448PrivateKey
    ):
        raise InvalidPrivateKeyFileError(
            f"{privatekey} is not a supported private key type"
        )
    if isinstance(
        privatekey,
        (
            MLDSA44PrivateKey,
            MLDSA65PrivateKey,
            MLDSA87PrivateKey,
        ),
    ):
        raise InvalidPrivateKeyFileError(
            f"{privatekey} is not a supported private key type"
        )
    return privatekey


def load_publickey(
    *, path: str | os.PathLike, key_format: KeySerializationFormat
) -> AllPublicKeyTypes:
    publickey_loaders = {
        "PEM": serialization.load_pem_public_key,
        "DER": serialization.load_der_public_key,
        "SSH": serialization.load_ssh_public_key,
    }

    try:
        publickey_loader = publickey_loaders[key_format]
    except KeyError as e:
        raise InvalidKeyFormatError(
            f"{key_format} is not a valid key format ({','.join(publickey_loaders)})"
        ) from e

    if not os.path.exists(path):
        raise InvalidPublicKeyFileError(f"No file was found at {path}")

    try:
        with open(path, "rb") as f:
            content = f.read()

            publickey = publickey_loader(
                data=content,
            )
    except ValueError as e:
        raise InvalidPublicKeyFileError(e) from e
    except UnsupportedAlgorithm as e:
        raise InvalidAlgorithmError(e) from e

    return publickey


def compare_publickeys(pk1: PublicKeyTypes, pk2: PublicKeyTypes) -> bool:
    a = isinstance(pk1, Ed25519PublicKey)
    b = isinstance(pk2, Ed25519PublicKey)
    if a or b:
        if not a or not b:
            return False
        a_bytes = pk1.public_bytes(
            serialization.Encoding.Raw, serialization.PublicFormat.Raw
        )
        b_bytes = pk2.public_bytes(
            serialization.Encoding.Raw, serialization.PublicFormat.Raw
        )
        return a_bytes == b_bytes
    return pk1.public_numbers() == pk2.public_numbers()  # type: ignore


def compare_encryption_algorithms(
    ea1: serialization.KeySerializationEncryption,
    ea2: serialization.KeySerializationEncryption,
) -> bool:
    if isinstance(ea1, serialization.NoEncryption) and isinstance(
        ea2, serialization.NoEncryption
    ):
        return True
    if isinstance(ea1, serialization.BestAvailableEncryption) and isinstance(
        ea2, serialization.BestAvailableEncryption
    ):
        return ea1.password == ea2.password
    return False


def get_encryption_algorithm(
    passphrase: bytes,
) -> serialization.KeySerializationEncryption:
    try:
        return serialization.BestAvailableEncryption(passphrase)
    except ValueError as e:
        raise InvalidPassphraseError(e) from e


def validate_comment(comment: str) -> None:
    if not hasattr(comment, "encode"):
        raise InvalidCommentError(f"{comment} cannot be encoded to text")


def extract_comment(path: str | os.PathLike) -> str:
    if not os.path.exists(path):
        raise InvalidPublicKeyFileError(f"No file was found at {path}")

    try:
        with open(path, "rb") as f:
            fields = f.read().split(b" ", 2)
            if len(fields) == 3:
                comment = fields[2].decode(_TEXT_ENCODING)
            else:
                comment = ""
    except (IOError, OSError) as e:
        raise InvalidPublicKeyFileError(e) from e

    return comment


def calculate_fingerprint(openssh_publickey: bytes) -> str:
    digest = hashes.Hash(hashes.SHA256())
    decoded_pubkey = b64decode(openssh_publickey.split(b" ")[1])
    digest.update(decoded_pubkey)

    value = b64encode(digest.finalize()).decode(encoding=_TEXT_ENCODING).rstrip("=")
    return f"SHA256:{value}"


__all__ = (
    "CRYPTOGRAPHY_VERSION",
    "HAS_OPENSSH_SUPPORT",
    "AsymmetricKeypair",
    "InvalidAlgorithmError",
    "InvalidCommentError",
    "InvalidDataError",
    "InvalidKeyFormatError",
    "InvalidKeySizeError",
    "InvalidKeyTypeError",
    "InvalidPassphraseError",
    "InvalidPrivateKeyFileError",
    "InvalidPublicKeyFileError",
    "InvalidSignatureError",
    "OpenSSHError",
    "OpensshKeypair",
    "calculate_fingerprint",
    "compare_encryption_algorithms",
    "compare_publickeys",
    "extract_comment",
    "get_encryption_algorithm",
    "load_privatekey",
    "load_publickey",
    "validate_comment",
)



#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos_ospf_interfaces config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.ospf_interfaces import (
    Ospf_interfacesTemplate,
)


class Ospf_interfaces(ResourceModule):
    """
    The nxos_ospf_interfaces config class
    """

    def __init__(self, module):
        super(Ospf_interfaces, self).__init__(
            empty_fact_val=[],
            facts_module=Facts(module),
            module=module,
            resource="ospf_interfaces",
            tmplt=Ospf_interfacesTemplate(),
        )
        self.parsers = [
            "authentication",
            "authentication_key",
            "message_digest_key",
            "cost",
            "dead_interval",
            "hello_interval",
            "instance",
            "mtu_ignore",
            "network",
            "passive_interface",
            "priority",
            "retransmit_interval",
            "shutdown",
            "transmit_delay",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {entry["name"]: entry for entry in self.want}
        haved = {entry["name"]: entry for entry in self.have}

        # turn all lists of dicts into dicts prior to merge
        for entry in wantd, haved:
            self._list_to_dict(entry)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self._compare(want={}, have=have)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Ospf_interfaces network resource.
        """
        begin = len(self.commands)
        self._compare_ospf_interfaces(want, have)
        if len(self.commands) != begin:
            self.commands.insert(begin, self._tmplt.render(want or have, "interface", False))

    def _compare_ospf_interfaces(self, want, have):
        waf = want.get("address_family", {})
        haf = have.get("address_family", {})

        for afi in ("ipv4", "ipv6"):
            witem = waf.pop(afi, {})
            hitem = haf.pop(afi, {})

            # this key needs to be compared separately and
            # popped from `authentication` dict to
            # preserve idempotence for other keys in this dict
            self.compare(["authentication.key_chain"], want=witem, have=hitem)
            witem.get("authentication", {}).pop("key_chain", None)
            hitem.get("authentication", {}).pop("key_chain", None)

            # this ensures that the "no" form of "ip ospf passive-interface"
            # command is executed even when there is no existing config
            if witem.get("passive_interface") is False and "passive_interface" not in hitem:
                hitem["passive_interface"] = True

            if "passive_interface" in hitem and witem.get("default_passive_interface"):
                self.commands.append(self._generate_passive_intf(witem))

            self.compare(parsers=self.parsers, want=witem, have=hitem)

            # compare top-level `multi_areas` config
            for area in witem.get("multi_areas", []):
                if area not in hitem.get("multi_areas", []):
                    self.addcmd({"afi": afi, "area": area}, "multi_areas", negate=False)
            # remove superfluous top-level `multi_areas` config
            for area in hitem.get("multi_areas", []):
                if area not in witem.get("multi_areas", []):
                    self.addcmd({"afi": afi, "area": area}, "multi_areas", negate=True)

            # compare config->address_family->processes
            self._compare_processes(afi, witem.get("processes", {}), hitem.get("processes", {}))

    def _compare_processes(self, afi, want, have):
        # add and update config->address_family->processes

        for w_id, wproc in want.items():
            hproc = have.pop(w_id, {})
            hproc["afi"] = wproc["afi"] = afi

            # compare config->address_family->processes->area
            self.compare(["area"], wproc, hproc)

            # compare config->address_family->processes->multi_areas
            marea_dict = {"afi": afi, "process_id": wproc["process_id"]}
            for area in wproc.get("multi_areas", []):
                if area not in hproc.get("multi_areas", []):
                    marea_dict["area"] = area
                    self.addcmd(marea_dict, "processes_multi_areas", negate=False)
            # remove superfluous processes->multi_areas config
            for area in hproc.get("multi_areas", []):
                if area not in wproc.get("multi_areas", []):
                    marea_dict["area"] = area
                    self.addcmd(marea_dict, "processes_multi_areas", negate=True)

        # remove superflous config->address_family->processes
        for hproc in have.values():
            hproc["afi"] = afi

            # remove config->address_family->processes->area
            self.addcmd(hproc, "area", negate=True)

            # remove superfluous processes->multi_areas config
            marea_dict = {"afi": afi, "process_id": hproc["process_id"]}
            for area in hproc.get("multi_areas", []):
                marea_dict["area"] = area
                self.addcmd(marea_dict, "processes_multi_areas", negate=True)

    def _list_to_dict(self, entry):
        for item in entry.values():
            for ag in item.get("address_family", []):
                ag["processes"] = {
                    subentry["process_id"]: subentry for subentry in ag.get("processes", [])
                }
            item["address_family"] = {
                subentry["afi"]: subentry for subentry in item.get("address_family", [])
            }

    def _generate_passive_intf(self, data):
        cmd = "default "
        if data["afi"] == "ipv4":
            cmd += "ip ospf passive-interface"
        else:
            cmd += "ospfv3 passive-interface"
        return cmd



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Gaspard Micol (@gmicol) <gmicol@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_tenant_custom_qos_policy
short_description: Manage Custom QoS Policies in Tenant Policy Templates on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Custom Quality of Service (QoS) Policies in Tenant Policy Templates on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Gaspard Micol (@gmicol)
options:
  template:
    description:
    - The name of the tenant template.
    type: str
    aliases: [ tenant_template ]
    required: true
  name:
    description:
    - The name of the Custom QoS Policy.
    type: str
    aliases: [ custom_qos_policy ]
  uuid:
    description:
    - The UUID of the Custom QoS Policy.
    - This parameter is required when the Custom QoS Policy O(name) needs to be updated.
    aliases: [ custom_qos_policy_uuid ]
    type: str
  description:
    description:
    - The description of the Custom QoS Policy.
    - Providing an empty string will remove the O(description="") from the Custom QoS Policy.
    type: str
  dscp_mappings:
    description:
    - The Differentiated Services Code Point (DSCP) mappings of the Custom QoS Policy.
    - Both O(dscp_mappings.dscp_from) and O(dscp_mappings.dscp_to) cannot be set to C(unspecified).
    - Providing a new list of O(dscp_mappings) will completely replace an existing one from the Custom QoS Policy.
    - Providing an empty list will remove the  O(cos_mappings=[]) from the Custom QoS Policy.
    type: list
    elements: dict
    suboptions:
      dscp_from:
        description:
        - The starting encoding point of the DSCP range.
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices:
          - af11
          - af12
          - af13
          - af21
          - af22
          - af23
          - af31
          - af32
          - af33
          - af41
          - af42
          - af43
          - cs0
          - cs1
          - cs2
          - cs3
          - cs4
          - cs5
          - cs6
          - cs7
          - expedited_forwarding
          - unspecified
          - voice_admit
        aliases: [ from ]
      dscp_to:
        description:
        - The ending encoding point of the DSCP range.
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices:
          - af11
          - af12
          - af13
          - af21
          - af22
          - af23
          - af31
          - af32
          - af33
          - af41
          - af42
          - af43
          - cs0
          - cs1
          - cs2
          - cs3
          - cs4
          - cs5
          - cs6
          - cs7
          - expedited_forwarding
          - unspecified
          - voice_admit
        aliases: [ to ]
      dscp_target:
        description:
        - The DSCP target encoding point for egressing traffic.
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices:
          - af11
          - af12
          - af13
          - af21
          - af22
          - af23
          - af31
          - af32
          - af33
          - af41
          - af42
          - af43
          - cs0
          - cs1
          - cs2
          - cs3
          - cs4
          - cs5
          - cs6
          - cs7
          - expedited_forwarding
          - unspecified
          - voice_admit
        aliases: [ target ]
      target_cos:
        description:
        - The target CoS value/traffic type for the egressing traffic.
        - Defaults to C(unspecified) when unset during creation.
        - Both CoS value and traffic type are allowed.
          For example, O(dscp_mappings.target_cos=cos0) or O(dscp_mappings.target_cos=background) are the same valid inputs.
        type: str
        choices:
          - background
          - cos0
          - best_effort
          - cos1
          - excellent_effort
          - cos2
          - critical_applications
          - cos3
          - video
          - cos4
          - voice
          - cos5
          - internetwork_control
          - cos6
          - network_control
          - cos7
          - unspecified
          - cos8
      qos_priority:
        description:
        - The QoS priority level to which the DSCP values will be mapped.
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
        aliases: [ priority, prio ]
  cos_mappings:
    description:
    - The Class of Service (CoS) mappings of the Custom QoS Policy.
    - Both O(cos_mappings.dot1p_from) and O(cos_mappings.dot1p_to) cannot be set to C(unspecified).
    - Providing a new list of O(cos_mappings) will completely replace an existing one from the Custom QoS Policy.
    - Providing an empty list will remove the O(cos_mappings=[]) from the Custom QoS Policy.
    type: list
    elements: dict
    suboptions:
      dot1p_from:
        description:
        - The starting value/traffic type of the CoS range.
        - Defaults to C(unspecified) when unset during creation.
        - Both CoS values and their respective traffic types are allowed.
          For example, O(cos_mappings.dot1p_from=cos0) or O(cos_mappings.dot1p_from=background) are the same valid inputs.
        type: str
        choices:
          - background
          - cos0
          - best_effort
          - cos1
          - excellent_effort
          - cos2
          - critical_applications
          - cos3
          - video
          - cos4
          - voice
          - cos5
          - internetwork_control
          - cos6
          - network_control
          - cos7
          - unspecified
          - cos8
        aliases: [ from ]
      dot1p_to:
        description:
        - The ending value/traffic type of the CoS range.
        - Defaults to C(unspecified) when unset during creation.
        - Both CoS value and traffic type are allowed.
          For example, O(cos_mappings.dot1p_to=cos0) or O(cos_mappings.dot1p_to=background) are the same valid inputs.
        type: str
        choices:
          - background
          - cos0
          - best_effort
          - cos1
          - excellent_effort
          - cos2
          - critical_applications
          - cos3
          - video
          - cos4
          - voice
          - cos5
          - internetwork_control
          - cos6
          - network_control
          - cos7
          - unspecified
          - cos8
        aliases: [ to ]
      dscp_target:
        description:
        - The DSCP target encoding point for egressing traffic.
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices:
          - af11
          - af12
          - af13
          - af21
          - af22
          - af23
          - af31
          - af32
          - af33
          - af41
          - af42
          - af43
          - cs0
          - cs1
          - cs2
          - cs3
          - cs4
          - cs5
          - cs6
          - cs7
          - expedited_forwarding
          - unspecified
          - voice_admit
        aliases: [ target ]
      target_cos:
        description:
        - The target CoS value/traffic type for the egressing traffic.
        - Defaults to C(unspecified) when unset during creation.
        - Both CoS values and their respective traffic types are allowed.
          For example, O(cos_mappings.target_cos=cos0) or O(cos_mappings.target_cos=background) are the same valid inputs.
        type: str
        choices:
          - background
          - cos0
          - best_effort
          - cos1
          - excellent_effort
          - cos2
          - critical_applications
          - cos3
          - video
          - cos4
          - voice
          - cos5
          - internetwork_control
          - cos6
          - network_control
          - cos7
          - unspecified
          - cos8
      qos_priority:
        description:
        - The QoS priority level to which the DSCP values will be mapped.
        - Defaults to C(unspecified) when unset during creation.
        type: str
        choices: [ level1, level2, level3, level4, level5, level6, unspecified ]
        aliases: [ priority, prio ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new Custom QoS Policy object
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: custom_qos_policy_1
    dscp_mappings:
      - dscp_from: af11
        dscp_to: af12
        dscp_target: af11
        target_cos: background
        qos_priority: level1
    cos_mappings:
      - dot1p_from: background
        dot1p_to: best_effort
        target: af11
        target_cos: background
        qos_priority: level1
    state: present
  register: custom_qos_policy_1

- name: Update a Custom QoS Policy object name with UUID
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: custom_qos_policy_2
    uuid: "{{ custom_qos_policy_1.current.uuid }}"
    state: present

- name: Query a Custom QoS Policy object with name
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: custom_qos_policy_1
    state: query
  register: query_name

- name: Query a Custom QoS Policy object with UUID
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    uuid: "{{ custom_qos_policy_1.current.uuid }}"
    state: query
  register: query_uuid

- name: Query all Custom QoS Policy objects in a Tenant Template
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    state: query
  register: query_all

- name: Delete a Custom QoS Policy object with name
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: custom_qos_policy_1
    state: absent

- name: Delete a Custom QoS Policy object with UUID
  cisco.mso.ndo_tenant_custom_qos_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    uuid: "{{ custom_qos_policy_1.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
from ansible_collections.cisco.mso.plugins.module_utils.constants import (
    TARGET_DSCP_MAP,
    TARGET_COS_MAP,
    DSCP_COS_KEY_MAP,
    QOS_LEVEL,
)


def main():
    COS_MAP_CHOICES = list(TARGET_COS_MAP) + list(TARGET_COS_MAP.values())
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["tenant_template"]),
        name=dict(type="str", aliases=["custom_qos_policy"]),
        uuid=dict(type="str", aliases=["custom_qos_policy_uuid"]),
        description=dict(type="str"),
        dscp_mappings=dict(
            type="list",
            elements="dict",
            options=dict(
                dscp_from=dict(type="str", choices=list(TARGET_DSCP_MAP), aliases=["from"]),
                dscp_to=dict(type="str", choices=list(TARGET_DSCP_MAP), aliases=["to"]),
                dscp_target=dict(type="str", choices=list(TARGET_DSCP_MAP), aliases=["target"]),
                target_cos=dict(type="str", choices=COS_MAP_CHOICES),
                qos_priority=dict(
                    type="str",
                    choices=QOS_LEVEL,
                    aliases=["priority", "prio"],
                ),
            ),
        ),
        cos_mappings=dict(
            type="list",
            elements="dict",
            options=dict(
                dot1p_from=dict(type="str", choices=COS_MAP_CHOICES, aliases=["from"]),
                dot1p_to=dict(type="str", choices=COS_MAP_CHOICES, aliases=["to"]),
                dscp_target=dict(type="str", choices=list(TARGET_DSCP_MAP), aliases=["target"]),
                target_cos=dict(type="str", choices=COS_MAP_CHOICES),
                qos_priority=dict(
                    type="str",
                    choices=QOS_LEVEL,
                    aliases=["priority", "prio"],
                ),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    dscp_mappings = module.params.get("dscp_mappings")
    if dscp_mappings:
        dscp_mappings = [
            {
                DSCP_COS_KEY_MAP.get(k): TARGET_DSCP_MAP.get(v) if k in ("dscp_from", "dscp_to", "dscp_target") else TARGET_COS_MAP.get(v, v)
                for k, v in d.items()
            }
            for d in dscp_mappings
        ]
    cos_mappings = module.params.get("cos_mappings")
    if cos_mappings:
        cos_mappings = [
            {DSCP_COS_KEY_MAP.get(k): TARGET_DSCP_MAP.get(v) if k == "dscp_target" else TARGET_COS_MAP.get(v, v) for k, v in d.items()} for d in cos_mappings
        ]
    state = module.params.get("state")

    template_object = MSOTemplate(mso, "tenant", template)
    template_object.validate_template("tenantPolicy")

    custom_qos_policies = template_object.template.get("tenantPolicyTemplate", {}).get("template", {}).get("qosPolicies", [])
    object_description = "Custom QoS Policy"
    custom_qos_policy_attrs_path = None
    match = None

    if state in ["query", "absent"] and custom_qos_policies == []:
        mso.exit_json()
    elif state == "query" and not (name or uuid):
        mso.existing = custom_qos_policies
    elif custom_qos_policies and (name or uuid):
        match = template_object.get_object_by_key_value_pairs(
            object_description, custom_qos_policies, [KVPair("uuid", uuid) if uuid else KVPair("name", name)]
        )
        if match:
            custom_qos_policy_attrs_path = "/tenantPolicyTemplate/template/qosPolicies/{0}".format(match.index)
            mso.existing = mso.previous = copy.deepcopy(match.details)

    ops = []

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        mso_values = dict(
            name=name,
            description=description,
            cosMappings=cos_mappings,
            dscpMappings=dscp_mappings,
        )

        if mso.existing and match:
            append_update_ops_data(ops, match.details, custom_qos_policy_attrs_path, mso_values)
            mso.sanitize(match.details, collate=True)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path="/tenantPolicyTemplate/template/qosPolicies/-", value=mso.sent))

    elif state == "absent":
        if mso.existing and match:
            ops.append(dict(op="remove", path=custom_qos_policy_attrs_path))

    if not module.check_mode and ops:
        response_object = mso.request(template_object.template_path, method="PATCH", data=ops)
        custom_qos_policies = response_object.get("tenantPolicyTemplate", {}).get("template", {}).get("qosPolicies", [])
        match = template_object.get_object_by_key_value_pairs(
            object_description, custom_qos_policies, [KVPair("uuid", uuid) if uuid else KVPair("name", name)]
        )
        if match:
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Hsrp_interfaces parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Hsrp_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Hsrp_interfacesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""^interface
                    (\s(?P<name>\S+))
                    $""",
                re.VERBOSE,
            ),
            "setval": "interface {{ name }}",
            "result": {"{{ name }}": {"name": "{{ name }}"}},
            "shared": True,
        },
        {
            "name": "mac_refresh",
            "getval": re.compile(
                r"""
                \s+standby\smac-refresh\s(?P<mac_refresh_number>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "standby mac-refresh {{ mac_refresh|string }}",
            "result": {
                "{{ name }}": {
                    "mac_refresh": "{{ mac_refresh_number }}",
                },
            },
        },
        {
            "name": "version",
            "getval": re.compile(
                r"""
                \s*standby\sversion\s(?P<version>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "standby version {{ version|string }}",
            "result": {
                "{{ name }}": {
                    "version": "{{ version }}",
                },
            },
        },
        {
            "name": "delay",
            "getval": re.compile(
                r"""
                \s+standby\sdelay
                (\sminimum\s(?P<minimum>\d+))?
                (\sreload\s(?P<reload>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "standby delay"
                      "{{ ' minimum ' + delay.minimum|string if delay.minimum is defined else ''}}"
                      "{{ ' reload ' + delay.reload|string if delay.reload is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "delay": {
                        "minimum": "{{ minimum }}",
                        "reload": "{{ reload }}",
                    },
                },
            },
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""
                \s+standby\sbfd\s
                $""", re.VERBOSE,
            ),
            "setval": "standby bfd",
            "result": {
                "{{ name }}": {
                    "bfd": True,
                },
            },
        },
        {
            "name": "use_bia",
            "getval": re.compile(
                r"""
                \s+standby\suse-bia
                (\s(?P<bia>\sscope\sinterface))?
                $""", re.VERBOSE,
            ),
            "remval": "standby use-bia",
            "setval": "standby use-bia"
            "{{ ' scope interface' if use_bia.scope|d(False) is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "use_bia": {
                        "set": True,
                        "scope": "{{ not not bia }}",
                    },
                },
            },
        },
        {
            "name": "timers",
            "getval": re.compile(
                r"""
                \s*standby\s(?P<grp_no>\d+)
                \s*timers
                (?:\smsec\s(?P<msec_hello_interval>\d+)|\s(?P<hello_interval>\d+))
                (?:\smsec\s(?P<msec_hold_time>\d+)|\s(?P<hold_time>\d+))
                $""", re.VERBOSE,
            ),
            "setval": "standby"
            "{{ ' ' + group_no|string if group_no is defined else ''}}"
            " timers"
            "{{ (' msec ' + timer.msec.hello_interval|string) "
            "if timer.msec.hello_interval is defined "
            "else (' ' + timer.hello_interval|string) "
            "if timer.hello_interval is defined else '' }}"
            "{{ (' msec ' + timer.msec.hold_time|string) "
            "if timer.msec.hold_time is defined "
            "else (' ' + timer.hold_time|string) "
            "if timer.hold_time is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "timers": {
                            "hello_interval": "{{ hello_interval }}",
                            "hold_time": "{{ hold_time }}",
                            "msec": {
                                "hello_interval": "{{ msec_hello_interval }}",
                                "hold_time": "{{ msec_hold_time }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "follow",
            "getval": re.compile(
                r"""
                \s*standby\s(?P<grp_no>\d+)
                \s*follow\s(?P<description>.+$)
                $""", re.VERBOSE,
            ),
            "setval": "standby {{ group_no|string if group_no is defined else '' }} follow {{ follow }}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "follow": "{{ description }}",
                    },
                },
            },
        },
        {
            "name": "priority",
            "getval": re.compile(
                r"""
                \s+standby\s*(?P<grp_no>\d+)?\s*priority\s(?P<priority>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "standby {{ group_no|string if group_no is defined else '' }} priority {{ priority|string }}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "priority": "{{ priority }}",
                    },
                },
            },
        },
        {
            "name": "preempt",
            "getval": re.compile(
                r"""
                \s+standby\s*(?P<grp_no>\d+)?\s*preempt
                (\s(?P<delay>delay))?
                (\sminimum\s(?P<minimum>\d+))?
                (\sreload\s(?P<reload>\d+))?
                (\ssync\s(?P<sync>\d+))?
                $""", re.VERBOSE,
            ),
            "remval": "standby "
                      "{{ group_no|string if group_no is defined else ''}}"
                      " preempt",
            "setval": "standby "
                      "{{ group_no|string if group_no is defined else ''}}"
                      " preempt"
                      "{{ ' delay' if preempt.delay|d(False) else ''}}"
                      "{{ ' minimum ' + preempt.minimum|string if preempt.minimum is defined else ''}}"
                      "{{ ' reload ' + preempt.reload|string if preempt.reload is defined else ''}}"
                      "{{ ' sync ' + preempt.sync|string if preempt.sync is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "preempt": {
                            "enabled": True,
                            "delay": "{{ not not delay }}",
                            "minimum": "{{ minimum }}",
                            "reload": "{{ reload }}",
                            "sync": "{{ sync }}",
                        },
                    },
                },
            },
        },
        {
            "name": "track",
            "getval": re.compile(
                r"""
                \s+standby\s*(?P<grp_no>\d+)\s*track\s(?P<track_no>\d+)
                (\sdecrement\s(?P<decrement>\d+))?
                (\s(?P<shutdown>shutdown))?
                $""", re.VERBOSE,
            ),
            "setval": "standby"
                      "{{ ' ' + track.group_no|string if track.group_no is defined else ''}}"
                      " track"
                      "{{ ' ' + track.track_no|string if track.track_no is defined else ''}}"
                      "{{ ' decrement ' + track.decrement|string if track.decrement is defined else ''}}"
                      "{{ ' shutdown' if track.shutdown|d(False) else ''}}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "track": [{
                            "track_no": "{{ track_no }}",
                            "decrement": "{{ decrement }}",
                            "shutdown": "{{ not not shutdown }}",
                        }],
                    },
                },
            },
        },
        {
            "name": "ip",
            "getval": re.compile(
                r"""
                \s+standby\s*(?P<grp_no>\d+)\s*ip\s+(?P<virtual_ip>\S+)
                (\s(?P<secondary>secondary))?
                $""", re.VERBOSE,
            ),
            "setval": "standby"
                      "{{ ' ' + ip.group_no|string if ip.group_no is defined else ''}}"
                      " ip"
                      "{{ ' ' + ip.virtual_ip if ip.virtual_ip is defined else ''}}"
                      "{{ ' secondary' if ip.secondary|d(False) else ''}}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "ip": [{
                            "virtual_ip": "{{ virtual_ip }}",
                            "secondary": "{{ not not secondary }}",
                        }],
                    },
                },
            },
        },
        {
            "name": "ipv6.autoconfig",
            "getval": re.compile(
                r"""
                \s*standby(\s+(?P<grp_no>\d+))\s+ipv6\sautoconfig
                $""", re.VERBOSE,
            ),
            "setval": "standby"
                      "{{ ' ' + group_no|string if group_no is defined else ''}}"
                      " ipv6"
                      "{{ ' autoconfig' if ipv6.autoconfig|d(False) else ''}}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "ipv6": {
                            "autoconfig": True,
                        },
                    },
                },
            },
        },
        {
            "name": "ipv6_addr",
            "getval": re.compile(
                r"""
                \s+standby(\s(?P<grp_no>\d+))\sipv6
                (\s(?P<address>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "standby"
                      "{{ ' ' + ipv6_addr.group_no|string if ipv6_addr.group_no is defined else ''}}"
                      " ipv6"
                      "{{ ' ' + ipv6_addr.address if ipv6_addr.address is defined else ''}}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "ipv6": {"addresses": ["{{ address }}"]},
                    },
                },
            },
        },
        {
            "name": "mac_address",
            "getval": re.compile(
                r"""
                \s+standby\s*(?P<grp_no>\d+)?\s*mac-address\s(?P<mac_address>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "standby {{ group_no|string if group_no is defined else '' }} mac-address {{ mac_address }}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "mac_address": "{{ mac_address }}",
                    },
                },
            },
        },
        {
            "name": "group_name",
            "getval": re.compile(
                r"""
                \s+standby\s*(?P<grp_no>\d+)?\s*name\s+(?P<group_name>.+)$
                """, re.VERBOSE,
            ),
            "setval": "standby {{ group_no|string if group_no is defined else '' }} name {{ group_name }}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "group_name": "{{ group_name }}",
                    },
                },
            },
        },
        {
            "name": "authentication",
            "getval": re.compile(
                r"""
                \s*standby\s(?P<grp_no>\d+)
                \s*authentication
                (\smd5\skey-chain\s(?P<key_chain>\S+)|
                \smd5\skey-string\s(?:(?P<encryption>\d+)\s)?(?P<key_string>\S+)|
                \s(?P<password_text>\S+))?
                (\stimeout\s(?P<timeout>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "standby"
            "{{ ' ' + group_no|string if group_no is defined else ''}}"
            " authentication"
            "{{ (' ' + authentication.password_text|string) "
            "if authentication.password_text is defined else '' }}"
            "{{ (' md5 key-chain ' + authentication.key_chain|string) "
            "if authentication.key_chain is defined else '' }}"
            "{{ (' md5 key-string ' + authentication.encryption|string + ' ' + authentication.key_string) "
            "if authentication.key_string is defined and authentication.encryption is defined "
            "else (' md5 key-string ' + authentication.key_string) "
            "if authentication.key_string is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "group_{{ grp_no|string }}": {
                        "authentication": {
                            "key_chain": "{{ key_chain }}",
                            "key_string": "'{{ key_string }}'",
                            "encryption": "{{ encryption|string }}",
                            "password_text": "{{ password_text }}",
                            "time_out": "{{ timeout }}",
                        },
                    },
                },
            },
        },
        {
            "name": "redirect.timers",
            "getval": re.compile(
                r"""
                \s+standby\sredirect\stimers\s(?P<adv_timer>\d+)\s(?P<holddown_timer>\d+)
                $""", re.VERBOSE,
            ),
            "compval": "redirect",
            "setval": "standby redirect timers {{ redirect.timers.adv_timer|string }} {{ redirect.timers.holddown_timer|string }}",
            "result": {
                "{{ name }}": {
                    "redirect": {
                        "timers": {
                            "adv_timer": "{{ adv_timer }}",
                            "holddown_timer": "{{ holddown_timer }}",
                        },
                    },
                },
            },
        },
        {
            "name": "redirect.advertisement.authentication",
            "getval": re.compile(
                r"""
                \s*standby\sredirect
                \sadvertisement\sauthentication
                (\smd5\skey-chain\s(?P<key_chain>\S+)|
                \smd5\skey-string\s(?:(?P<encryption>\d+)\s)?(?P<key_string>\S+))?
                (\stimeout\s(?P<timeout>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "standby redirect advertisement authentication"
            "{{ (' md5 key-chain ' + redirect.advertisement.authentication.key_chain|string) "
            "if redirect.advertisement.authentication.key_chain is defined else '' }}"
            "{{ (' md5 key-string ' + "
            "(redirect.advertisement.authentication.encryption|string + ' ') "
            "if redirect.advertisement.authentication.encryption is defined else '' "
            "+ redirect.advertisement.authentication.key_string|string) "
            "if redirect.advertisement.authentication.key_string is defined else '' }}",
            "result": {
                "{{ name }}": {
                    "redirect": {
                        "advertisement": {
                            "authentication": {
                                "key_chain": "{{ key_chain }}",
                                "key_string": "'{{ key_string }}'",
                                "encryption": "{{ encryption }}",
                                "time_out": "{{ timeout }}",
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python

# Copyright (c) Ansible project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_identity_provider

short_description: Allows administration of Keycloak identity providers using Keycloak API

version_added: 3.6.0

description:
  - This module allows you to add, remove or modify Keycloak identity providers using the Keycloak REST API. It requires access
    to the REST API using OpenID Connect; the user connecting and the client being used must have the requisite access rights.
    In a default Keycloak installation, admin-cli and an admin user would work, as would a separate client definition with
    the scope tailored to your needs and a user having the expected roles.
  - The names of module options are snake_cased versions of the camelCase ones found in the Keycloak API and its documentation
    at U(https://www.keycloak.org/docs-api/15.0/rest-api/index.html).
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  action_group:
    version_added: 10.2.0

options:
  state:
    description:
      - State of the identity provider.
      - On V(present), the identity provider is created if it does not yet exist, or updated with the parameters you provide.
      - On V(absent), the identity provider is removed if it exists.
    default: 'present'
    type: str
    choices:
      - present
      - absent

  realm:
    description:
      - The Keycloak realm under which this identity provider resides.
    default: 'master'
    type: str

  alias:
    description:
      - The alias uniquely identifies an identity provider and it is also used to build the redirect URI.
    required: true
    type: str

  display_name:
    description:
      - Friendly name for identity provider.
    aliases:
      - displayName
    type: str

  enabled:
    description:
      - Enable/disable this identity provider.
    type: bool

  store_token:
    description:
      - Enable/disable whether tokens must be stored after authenticating users.
    aliases:
      - storeToken
    type: bool

  add_read_token_role_on_create:
    description:
      - Enable/disable whether new users can read any stored tokens. This assigns the C(broker.read-token) role.
    aliases:
      - addReadTokenRoleOnCreate
    type: bool

  trust_email:
    description:
      - If enabled, email provided by this provider is not verified even if verification is enabled for the realm.
    aliases:
      - trustEmail
    type: bool

  link_only:
    description:
      - If true, users cannot log in through this provider. They can only link to this provider. This is useful if you do
        not want to allow login from the provider, but want to integrate with a provider.
    aliases:
      - linkOnly
    type: bool

  first_broker_login_flow_alias:
    description:
      - Alias of authentication flow, which is triggered after first login with this identity provider.
    aliases:
      - firstBrokerLoginFlowAlias
    type: str

  post_broker_login_flow_alias:
    description:
      - Alias of authentication flow, which is triggered after each login with this identity provider.
    aliases:
      - postBrokerLoginFlowAlias
    type: str

  authenticate_by_default:
    description:
      - Specifies if this identity provider should be used by default for authentication even before displaying login screen.
    aliases:
      - authenticateByDefault
    type: bool

  provider_id:
    description:
      - Protocol used by this provider (supported values are V(oidc) or V(saml)).
    aliases:
      - providerId
    type: str

  config:
    description:
      - Dict specifying the configuration options for the provider; the contents differ depending on the value of O(provider_id).
        Examples are given below for V(oidc) and V(saml). It is easiest to obtain valid config values by dumping an already-existing
        identity provider configuration through check-mode in the RV(existing) field.
    type: dict
    suboptions:
      hide_on_login_page:
        description:
          - If hidden, login with this provider is possible only if requested explicitly, for example using the C(kc_idp_hint)
            parameter.
        aliases:
          - hideOnLoginPage
        type: bool

      gui_order:
        description:
          - Number defining order of the provider in GUI (for example, on Login page).
        aliases:
          - guiOrder
        type: int

      sync_mode:
        description:
          - Default sync mode for all mappers. The sync mode determines when user data is synced using the mappers.
        aliases:
          - syncMode
        type: str

      issuer:
        description:
          - The issuer identifier for the issuer of the response. If not provided, no validation is performed.
        type: str

      authorizationUrl:
        description:
          - The Authorization URL.
        type: str

      tokenUrl:
        description:
          - The Token URL.
        type: str

      logoutUrl:
        description:
          - End session endpoint to use to logout user from external IDP.
        type: str

      userInfoUrl:
        description:
          - The User Info URL.
        type: str

      clientAuthMethod:
        description:
          - The client authentication method.
        type: str

      clientId:
        description:
          - The client or client identifier registered within the identity provider.
        type: str

      clientSecret:
        description:
          - The client or client secret registered within the identity provider.
        type: str

      defaultScope:
        description:
          - The scopes to be sent when asking for authorization.
        type: str

      validateSignature:
        description:
          - Enable/disable signature validation of external IDP signatures.
        type: bool

      useJwksUrl:
        description:
          - If V(true), identity provider public keys are downloaded from given JWKS URL.
        type: bool

      jwksUrl:
        description:
          - URL where identity provider keys in JWK format are stored. See JWK specification for more details.
        type: str

      entityId:
        description:
          - The Entity ID that is used to uniquely identify this SAML Service Provider.
        type: str

      singleSignOnServiceUrl:
        description:
          - The URL that must be used to send authentication requests (SAML AuthnRequest).
        type: str

      singleLogoutServiceUrl:
        description:
          - The URL that must be used to send logout requests.
        type: str

      backchannelSupported:
        description:
          - Does the external IDP support backchannel logout?
        type: str

      nameIDPolicyFormat:
        description:
          - Specifies the URI reference corresponding to a name identifier format.
        type: str

      principalType:
        description:
          - Way to identify and track external users from the assertion.
        type: str

      fromUrl:
        description:
          - IDP well-known OpenID Connect configuration URL.
          - Support only O(provider_id=oidc).
          - O(config.fromUrl) is mutually exclusive with O(config.userInfoUrl), O(config.authorizationUrl),
            O(config.tokenUrl), O(config.logoutUrl), O(config.issuer) and O(config.jwksUrl).
        type: str
        version_added: '11.2.0'

  mappers:
    description:
      - A list of dicts defining mappers associated with this Identity Provider.
    type: list
    elements: dict
    suboptions:
      id:
        description:
          - Unique ID of this mapper.
        type: str

      name:
        description:
          - Name of the mapper.
        type: str

      identityProviderAlias:
        description:
          - Alias of the identity provider for this mapper.
        type: str

      identityProviderMapper:
        description:
          - Type of mapper.
        type: str

      config:
        description:
          - Dict specifying the configuration options for the mapper; the contents differ depending on the value of O(mappers[].identityProviderMapper).
        type: dict

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Laurent Paumier (@laurpaum)
"""

EXAMPLES = r"""
- name: Create OIDC identity provider, authentication with credentials
  community.general.keycloak_identity_provider:
    state: present
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: admin
    auth_password: admin
    realm: myrealm
    alias: oidc-idp
    display_name: OpenID Connect IdP
    enabled: true
    provider_id: oidc
    config:
      issuer: https://idp.example.com
      authorizationUrl: https://idp.example.com/auth
      tokenUrl: https://idp.example.com/token
      userInfoUrl: https://idp.example.com/userinfo
      clientAuthMethod: client_secret_post
      clientId: my-client
      clientSecret: secret
      syncMode: FORCE
    mappers:
      - name: first_name
        identityProviderMapper: oidc-user-attribute-idp-mapper
        config:
          claim: first_name
          user.attribute: first_name
          syncMode: INHERIT
      - name: last_name
        identityProviderMapper: oidc-user-attribute-idp-mapper
        config:
          claim: last_name
          user.attribute: last_name
          syncMode: INHERIT

- name: Create OIDC identity provider, with well-known configuration URL
  community.general.keycloak_identity_provider:
    state: present
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: admin
    auth_password: admin
    realm: myrealm
    alias: oidc-idp
    display_name: OpenID Connect IdP
    enabled: true
    provider_id: oidc
    config:
      fromUrl: https://the-idp.example.com/auth/realms/idprealm/.well-known/openid-configuration
      clientAuthMethod: client_secret_post
      clientId: my-client
      clientSecret: secret

- name: Create SAML identity provider, authentication with credentials
  community.general.keycloak_identity_provider:
    state: present
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: admin
    auth_password: admin
    realm: myrealm
    alias: saml-idp
    display_name: SAML IdP
    enabled: true
    provider_id: saml
    config:
      entityId: https://auth.example.com/auth/realms/myrealm
      singleSignOnServiceUrl: https://idp.example.com/login
      wantAuthnRequestsSigned: true
      wantAssertionsSigned: true
    mappers:
      - name: roles
        identityProviderMapper: saml-user-attribute-idp-mapper
        config:
          user.attribute: roles
          attribute.friendly.name: User Roles
          attribute.name: roles
          syncMode: INHERIT

- name: Create OIDC identity provider, authentication with credentials and advanced claim to group
  community.general.keycloak_identity_provider:
    state: present
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: admin
    auth_password: admin
    realm: myrealm
    alias: oidc-idp
    display_name: OpenID Connect IdP
    enabled: true
    provider_id: oidc
    config:
      issuer: https://idp.example.com
      authorizationUrl: https://idp.example.com/auth
      tokenUrl: https://idp.example.com/token
      userInfoUrl: https://idp.example.com/userinfo
      clientAuthMethod: client_secret_post
      clientId: my-client
      clientSecret: secret
      syncMode: FORCE
    mappers:
      - name: group_name
        identityProviderMapper: oidc-advanced-group-idp-mapper
        config:
          claims: '[{"key":"my_key","value":"my_value"}]'
          group: group_name
          syncMode: INHERIT
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str
  sample: "Identity provider my-idp has been created"

proposed:
  description: Representation of proposed identity provider.
  returned: always
  type: dict
  sample:
    {
      "config": {
        "authorizationUrl": "https://idp.example.com/auth",
        "clientAuthMethod": "client_secret_post",
        "clientId": "my-client",
        "clientSecret": "secret",
        "issuer": "https://idp.example.com",
        "tokenUrl": "https://idp.example.com/token",
        "userInfoUrl": "https://idp.example.com/userinfo"
      },
      "displayName": "OpenID Connect IdP",
      "providerId": "oidc"
    }

existing:
  description: Representation of existing identity provider.
  returned: always
  type: dict
  sample:
    {
      "addReadTokenRoleOnCreate": false,
      "alias": "my-idp",
      "authenticateByDefault": false,
      "config": {
        "authorizationUrl": "https://old.example.com/auth",
        "clientAuthMethod": "client_secret_post",
        "clientId": "my-client",
        "clientSecret": "**********",
        "issuer": "https://old.example.com",
        "syncMode": "FORCE",
        "tokenUrl": "https://old.example.com/token",
        "userInfoUrl": "https://old.example.com/userinfo"
      },
      "displayName": "OpenID Connect IdP",
      "enabled": true,
      "firstBrokerLoginFlowAlias": "first broker login",
      "internalId": "4d28d7e3-1b80-45bb-8a30-5822bf55aa1c",
      "linkOnly": false,
      "providerId": "oidc",
      "storeToken": false,
      "trustEmail": false
    }

end_state:
  description: Representation of identity provider after module execution.
  returned: on success
  type: dict
  sample:
    {
      "addReadTokenRoleOnCreate": false,
      "alias": "my-idp",
      "authenticateByDefault": false,
      "config": {
        "authorizationUrl": "https://idp.example.com/auth",
        "clientAuthMethod": "client_secret_post",
        "clientId": "my-client",
        "clientSecret": "**********",
        "issuer": "https://idp.example.com",
        "tokenUrl": "https://idp.example.com/token",
        "userInfoUrl": "https://idp.example.com/userinfo"
      },
      "displayName": "OpenID Connect IdP",
      "enabled": true,
      "firstBrokerLoginFlowAlias": "first broker login",
      "internalId": "4d28d7e3-1b80-45bb-8a30-5822bf55aa1c",
      "linkOnly": false,
      "providerId": "oidc",
      "storeToken": false,
      "trustEmail": false
    }
"""

from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    camel,
    get_token,
    keycloak_argument_spec,
)


def sanitize(idp):
    idpcopy = deepcopy(idp)
    if "config" in idpcopy:
        if "clientSecret" in idpcopy["config"]:
            idpcopy["config"]["clientSecret"] = "**********"
    return idpcopy


def get_identity_provider_with_mappers(kc, alias, realm):
    idp = kc.get_identity_provider(alias, realm)
    if idp is not None:
        idp["mappers"] = sorted(kc.get_identity_provider_mappers(alias, realm), key=lambda x: x.get("name"))
        # clientSecret returned by API when using `get_identity_provider(alias, realm)` is always **********
        # to detect changes to the secret, we get the actual cleartext secret from the full realm info
        if "config" in idp:
            if "clientSecret" in idp["config"]:
                for idp_from_realm in kc.get_realm_by_id(realm).get("identityProviders", []):
                    if idp_from_realm["internalId"] == idp["internalId"]:
                        cleartext_secret = idp_from_realm.get("config", {}).get("clientSecret")
                        if cleartext_secret:
                            idp["config"]["clientSecret"] = cleartext_secret
    if idp is None:
        idp = {}
    return idp


def fetch_identity_provider_wellknown_config(kc, config):
    """
    Fetches OpenID Connect well-known configuration from a given URL and updates the config dict with discovered endpoints.
    Support for oidc providers only.
    :param kc: KeycloakAPI instance used to fetch endpoints and handle errors.
    :param config: Dictionary containing identity provider configuration, must include 'fromUrl' key to trigger fetch.
    :return: None. The config dict is updated in-place.
    """
    if config and "fromUrl" in config:
        if "providerId" in config and config["providerId"] != "oidc":
            kc.module.fail_json(msg="Only 'oidc' provider_id is supported when using 'fromUrl'.")
        endpoints = ["userInfoUrl", "authorizationUrl", "tokenUrl", "logoutUrl", "issuer", "jwksUrl"]
        if any(k in config for k in endpoints):
            kc.module.fail_json(
                msg="Cannot specify both 'fromUrl' and 'userInfoUrl', 'authorizationUrl', 'tokenUrl', 'logoutUrl', 'issuer' or 'jwksUrl'."
            )
        openIdConfig = kc.fetch_idp_endpoints_import_config_url(
            fromUrl=config["fromUrl"], realm=kc.module.params.get("realm", "master")
        )
        for k in endpoints:
            if k in openIdConfig:
                config[k] = openIdConfig[k]
        del config["fromUrl"]


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    mapper_spec = dict(
        id=dict(type="str"),
        name=dict(type="str"),
        identityProviderAlias=dict(type="str"),
        identityProviderMapper=dict(type="str"),
        config=dict(type="dict"),
    )

    meta_args = dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        realm=dict(type="str", default="master"),
        alias=dict(type="str", required=True),
        add_read_token_role_on_create=dict(type="bool", aliases=["addReadTokenRoleOnCreate"]),
        authenticate_by_default=dict(type="bool", aliases=["authenticateByDefault"]),
        config=dict(type="dict"),
        display_name=dict(type="str", aliases=["displayName"]),
        enabled=dict(type="bool"),
        first_broker_login_flow_alias=dict(type="str", aliases=["firstBrokerLoginFlowAlias"]),
        link_only=dict(type="bool", aliases=["linkOnly"]),
        post_broker_login_flow_alias=dict(type="str", aliases=["postBrokerLoginFlowAlias"]),
        provider_id=dict(type="str", aliases=["providerId"]),
        store_token=dict(type="bool", aliases=["storeToken"]),
        trust_email=dict(type="bool", aliases=["trustEmail"]),
        mappers=dict(type="list", elements="dict", options=mapper_spec),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    result = dict(changed=False, msg="", diff={}, proposed={}, existing={}, end_state={})

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    realm = module.params.get("realm")
    alias = module.params.get("alias")
    state = module.params.get("state")
    config = module.params.get("config")

    fetch_identity_provider_wellknown_config(kc, config)

    # Filter and map the parameters names that apply to the identity provider.
    idp_params = [
        x
        for x in module.params
        if x not in list(keycloak_argument_spec().keys()) + ["state", "realm", "mappers"]
        and module.params.get(x) is not None
    ]

    # See if it already exists in Keycloak
    before_idp = get_identity_provider_with_mappers(kc, alias, realm)

    # Build a proposed changeset from parameters given to this module
    changeset = {}

    for param in idp_params:
        new_param_value = module.params.get(param)
        old_value = before_idp[camel(param)] if camel(param) in before_idp else None
        if new_param_value != old_value:
            changeset[camel(param)] = new_param_value

    # special handling of mappers list to allow change detection
    if module.params.get("mappers") is not None:
        for change in module.params["mappers"]:
            change = {k: v for k, v in change.items() if v is not None}
            if change.get("id") is None and change.get("name") is None:
                module.fail_json(msg="Either `name` or `id` has to be specified on each mapper.")
            if before_idp == dict():
                old_mapper = dict()
            elif change.get("id") is not None:
                old_mapper = kc.get_identity_provider_mapper(change["id"], alias, realm)
                if old_mapper is None:
                    old_mapper = dict()
            else:
                found = [x for x in kc.get_identity_provider_mappers(alias, realm) if x["name"] == change["name"]]
                if len(found) == 1:
                    old_mapper = found[0]
                else:
                    old_mapper = dict()
            new_mapper = old_mapper.copy()
            new_mapper.update(change)

            if changeset.get("mappers") is None:
                changeset["mappers"] = list()
            # eventually this holds all desired mappers, unchanged, modified and newly added
            changeset["mappers"].append(new_mapper)

        # ensure idempotency in case module.params.mappers is not sorted by name
        changeset["mappers"] = sorted(
            changeset["mappers"], key=lambda x: x.get("id") if x.get("name") is None else x["name"]
        )

    # Prepare the desired values using the existing values (non-existence results in a dict that is save to use as a basis)
    desired_idp = before_idp.copy()
    desired_idp.update(changeset)

    result["proposed"] = sanitize(changeset)
    result["existing"] = sanitize(before_idp)

    # Cater for when it doesn't exist (an empty dict)
    if not before_idp:
        if state == "absent":
            # Do nothing and exit
            if module._diff:
                result["diff"] = dict(before="", after="")
            result["changed"] = False
            result["end_state"] = {}
            result["msg"] = "Identity provider does not exist; doing nothing."
            module.exit_json(**result)

        # Process a creation
        result["changed"] = True

        if module._diff:
            result["diff"] = dict(before="", after=sanitize(desired_idp))

        if module.check_mode:
            module.exit_json(**result)

        # create it
        desired_idp = desired_idp.copy()
        mappers = desired_idp.pop("mappers", [])
        kc.create_identity_provider(desired_idp, realm)
        for mapper in mappers:
            if mapper.get("identityProviderAlias") is None:
                mapper["identityProviderAlias"] = alias
            kc.create_identity_provider_mapper(mapper, alias, realm)
        after_idp = get_identity_provider_with_mappers(kc, alias, realm)

        result["end_state"] = sanitize(after_idp)

        result["msg"] = f"Identity provider {alias} has been created"
        module.exit_json(**result)

    else:
        if state == "present":
            # Process an update

            # no changes
            if desired_idp == before_idp:
                result["changed"] = False
                result["end_state"] = sanitize(desired_idp)
                result["msg"] = f"No changes required to identity provider {alias}."
                module.exit_json(**result)

            # doing an update
            result["changed"] = True

            if module._diff:
                result["diff"] = dict(before=sanitize(before_idp), after=sanitize(desired_idp))

            if module.check_mode:
                module.exit_json(**result)

            # do the update
            desired_idp = desired_idp.copy()
            updated_mappers = desired_idp.pop("mappers", [])
            original_mappers = list(before_idp.get("mappers", []))

            kc.update_identity_provider(desired_idp, realm)
            for mapper in updated_mappers:
                if mapper.get("id") is not None:
                    # only update existing if there is a change
                    for i, orig in enumerate(original_mappers):
                        if mapper["id"] == orig["id"]:
                            del original_mappers[i]
                            if mapper != orig:
                                kc.update_identity_provider_mapper(mapper, alias, realm)
                else:
                    if mapper.get("identityProviderAlias") is None:
                        mapper["identityProviderAlias"] = alias
                    kc.create_identity_provider_mapper(mapper, alias, realm)
            for mapper in [
                x for x in before_idp["mappers"] if [y for y in updated_mappers if y["name"] == x["name"]] == []
            ]:
                kc.delete_identity_provider_mapper(mapper["id"], alias, realm)

            after_idp = get_identity_provider_with_mappers(kc, alias, realm)

            result["end_state"] = sanitize(after_idp)

            result["msg"] = f"Identity provider {alias} has been updated"
            module.exit_json(**result)

        elif state == "absent":
            # Process a deletion
            result["changed"] = True

            if module._diff:
                result["diff"] = dict(before=sanitize(before_idp), after="")

            if module.check_mode:
                module.exit_json(**result)

            # delete it
            kc.delete_identity_provider(alias, realm)

            result["end_state"] = {}

            result["msg"] = f"Identity provider {alias} has been deleted"

    module.exit_json(**result)


if __name__ == "__main__":
    main()



# !/usr/bin/python
# Copyright Jakub Danek <danek.ja@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or
# https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_realm_localization

short_description: Allows management of Keycloak realm localization overrides via the Keycloak API

version_added: 12.4.0

description:
  - This module allows you to manage per-locale message overrides for a Keycloak realm using the Keycloak Admin REST API.
  - Requires access via OpenID Connect; the connecting user/client must have sufficient privileges.
  - The names of module options are snake_cased versions of the names found in the Keycloak API.

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full

options:
  force:
    description:
      - If V(false), only the keys listed in the O(overrides) are modified by this module. Any other pre-existing
        keys are ignored.
      - If V(true), all locale overrides are made to match configuration of this module. For example any keys
        missing from the O(overrides) are removed regardless of O(state) value.
    type: bool
    default: false
  locale:
    description:
      - Locale code for which the overrides apply (for example, V(en), V(fi), V(de)).
    type: str
    required: true
  parent_id:
    description:
      - Name of the realm that owns the locale overrides.
    type: str
    required: true
  state:
    description:
      - Desired state of localization overrides for the given locale.
      - On V(present), the set of overrides for the locale are made to match O(overrides).
        If O(force) is V(true) keys not listed in O(overrides) are removed,
        and the listed keys are created or updated.
        If O(force) is V(false)  keys not listed in O(overrides) are ignored,
        and the listed keys are created or updated.
      - On V(absent), overrides for the locale is removed. If O(force) is V(true), all keys are removed.
        If O(force) is V(false), only the keys listed in O(overrides) are removed.
    type: str
    choices: ['present', 'absent']
    default: present
  overrides:
    description:
      - List of overrides to ensure for the locale when O(state=present). Each item is a mapping with
        the record's O(overrides[].key) and its O(overrides[].value).
      - Ignored when O(state=absent).
    type: list
    elements: dict
    default: []
    suboptions:
      key:
        description:
          - The message key to override.
        type: str
        required: true
      value:
        description:
          - The override value for the message key. If omitted, value defaults to an empty string.
        type: str
        default: ""
        required: false

seealso:
  - module: community.general.keycloak_realm
    description: You can specify list of supported locales using O(community.general.keycloak_realm#module:supported_locales).

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author: Jakub Danek (@danekja)
"""

EXAMPLES = r"""
- name: Replace all overrides for locale "en" (credentials auth)
  community.general.keycloak_realm_localization:
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    parent_id: my-realm
    locale: en
    state: present
    force: true
    overrides:
      - key: greeting
        value: "Hello"
      - key: farewell
        value: "Bye"
  delegate_to: localhost

- name: Replace listed overrides for locale "en" (credentials auth)
  community.general.keycloak_realm_localization:
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    parent_id: my-realm
    locale: en
    state: present
    force: false
    overrides:
      - key: greeting
        value: "Hello"
      - key: farewell
        value: "Bye"
  delegate_to: localhost

- name: Ensure only one override exists for locale "fi" (token auth)
  community.general.keycloak_realm_localization:
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    token: TOKEN
    parent_id: my-realm
    locale: fi
    state: present
    force: true
    overrides:
      - key: app.title
        value: "Sovellukseni"
  delegate_to: localhost

- name: Remove all overrides for locale "de"
  community.general.keycloak_realm_localization:
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    parent_id: my-realm
    locale: de
    state: absent
    force: true
  delegate_to: localhost

- name: Remove only the listed overrides for locale "de"
  community.general.keycloak_realm_localization:
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    parent_id: my-realm
    locale: de
    state: absent
    force: false
    overrides:
      - key: app.title
      - key: foo
      - key: bar
  delegate_to: localhost
"""

RETURN = r"""
end_state:
  description:
    - Final state of localization overrides for the locale after module execution.
    - Contains the O(locale) and the list of O(overrides) as key/value items.
  returned: on success
  type: dict
  contains:
    locale:
      description: The locale code affected.
      type: str
      sample: en
    overrides:
      description: The list of overrides that exist after execution.
      type: list
      elements: dict
      sample:
        - key: greeting
          value: Hello
        - key: farewell
          value: Bye
"""

from copy import deepcopy

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    get_token,
    keycloak_argument_spec,
)


def _normalize_overrides(current: dict | None) -> list[dict]:
    """
    Accepts:
      - dict: {'k1': 'v1', ...}
    Return a sorted list of {'key', 'value'}.

    This helper provides a consistent shape for downstream comparison/diff logic.
    """
    if not current:
        return []

    return [{"key": k, "value": v} for k, v in sorted(current.items())]


def main():
    argument_spec = keycloak_argument_spec()

    # Single override record structure
    overrides_spec = dict(
        key=dict(type="str", no_log=False, required=True),
        value=dict(type="str", default=""),
    )

    meta_args = dict(
        locale=dict(type="str", required=True),
        parent_id=dict(type="str", required=True),
        state=dict(type="str", default="present", choices=["present", "absent"]),
        overrides=dict(type="list", elements="dict", options=overrides_spec, default=[]),
        force=dict(type="bool", default=False),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=([["token", "auth_realm", "auth_username", "auth_password"]]),
        required_together=([["auth_realm", "auth_username", "auth_password"]]),
    )

    result = dict(changed=False, msg="", end_state={}, diff=dict(before={}, after={}))

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    # Convenience locals for frequently used parameters
    locale = module.params["locale"]
    state = module.params["state"]
    parent_id = module.params["parent_id"]
    force = module.params["force"]

    desired_raw = module.params["overrides"]
    desired_overrides = _normalize_overrides({r["key"]: r.get("value") for r in desired_raw})

    old_overrides = _normalize_overrides(kc.get_localization_values(locale, parent_id) or {})
    before = {
        "locale": locale,
        "overrides": deepcopy(old_overrides),
    }

    # Proposed state used for diff reporting
    changeset = {
        "locale": locale,
        "overrides": [],
    }

    result["changed"] = False

    if state == "present":
        changeset["overrides"] = deepcopy(desired_overrides)

        # Compute two sets:
        # - to_update: keys missing or with different values
        # - to_remove: keys existing in current state but not in desired
        to_update = []
        to_remove = deepcopy(old_overrides)

        # Mark updates and remove matched ones from to_remove
        for record in desired_overrides:
            override_found = False

            for override in to_remove:
                if override["key"] == record["key"]:
                    override_found = True

                    # Value differs -> update needed
                    if override["value"] != record["value"]:
                        result["changed"] = True
                        to_update.append(record)

                    # Remove processed item so what's left in to_remove are deletions
                    to_remove.remove(override)
                    break

            if not override_found:
                # New key, must be created
                to_update.append(record)
                result["changed"] = True

        # ignore any left-overs in to_remove, force is false
        if not force:
            changeset["overrides"].extend(to_remove)
            to_remove = []

        if to_remove:
            result["changed"] = True

        if result["changed"]:
            if module._diff:
                result["diff"] = dict(before=before, after=changeset)

            if module.check_mode:
                result["msg"] = f"Locale {locale} overrides would be updated."

            else:
                for override in to_remove:
                    kc.delete_localization_value(locale, override["key"], parent_id)

                for override in to_update:
                    kc.set_localization_value(locale, override["key"], override["value"], parent_id)

                result["msg"] = f"Locale {locale} overrides have been updated."

        else:
            result["msg"] = f"Locale {locale} overrides are in sync."

        # For accurate end_state, read back from API unless we are in check_mode
        if not module.check_mode:
            final_overrides = _normalize_overrides(kc.get_localization_values(locale, parent_id) or {})

        else:
            final_overrides = ["overrides"]

        result["end_state"] = {"locale": locale, "overrides": final_overrides}

    elif state == "absent":
        if force:
            to_remove = old_overrides

        else:
            # touch only overrides listed in parameters, leave the rest be
            to_remove = deepcopy(desired_overrides)
            to_keep = deepcopy(old_overrides)

            for override in to_remove:
                found = False
                for keep in to_keep:
                    if override["key"] == keep["key"]:
                        to_keep.remove(keep)
                        found = True
                        break

                if not found:
                    to_remove.remove(override)

            changeset["overrides"] = to_keep

        if to_remove:
            result["changed"] = True

        if module._diff:
            result["diff"] = dict(before=before, after=changeset)

        if module.check_mode:
            if result["changed"]:
                result["msg"] = f"{len(to_remove)} overrides for locale {locale} would be deleted."
            else:
                result["msg"] = f"No overrides for locale {locale} to be deleted."

        else:
            for override in to_remove:
                kc.delete_localization_value(locale, override["key"], parent_id)

            if result["changed"]:
                result["msg"] = f"{len(to_remove)} overrides for locale {locale} deleted."
            else:
                result["msg"] = f"No overrides for locale {locale} to be deleted."

        result["end_state"] = changeset

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerregistrytokenpassword
version_added: "3.7.0"
short_description: Generate or  renegenerate the Azure Container Registry Token Password
description:
    - Generate or  renegenerate the Azure Container Registry Token Password.
    - Note that the generated password cannot be retrieved. Please store your credentials safely after generation.

options:
    resource_group:
        description:
            - The name of the resource group to which the container registry belongs.
        type: str
        required: true
    registry_name:
        description:
            - The name of the container registry.
        type: str
        required: true
    token_id:
        description:
            - The ID of the container registry token.
        type: str
        required: true
    expiry:
        description:
            - The expiry datetime of the password.
            - Sample as 2025/07/25.
            - If not configured, it will never expire.
        type: str
    name:
        description:
            - The name of the password.
            - If not configured, both passwords will be regenerated.
        type: str
        choices:
            - password1
            - password2

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Generate a new password for container registry token
  azure_rm_containerregistrytokenpassword:
    resource_group: myResourceGroup
    registry_name: myRegistry
    name: mytoken
    expiry: 2025/08/01
    token_id: token_id
'''

RETURN = '''
passwords:
    description:
        - A list of dictionaries containing facts for token passwords facts.
    returned: always
    type: list
    sample: [
       {
            "creation_time": "2025-07-24T08:28:50.569062Z",
            "expiry": "2025-08-01T00:00:00.000Z",
            "name": "password1"
        },
        {
            "creation_time": "2025-07-24T08:29:19.12587Z",
            "expiry": "2025-08-01T00:00:00.000Z",
            "name": "password2",
            "value": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        }
    ]
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.polling import LROPoller
    from datetime import datetime
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMContainerRegistryTokenPassword(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True,
            ),
            registry_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                choices=['password1', 'password2']
            ),
            token_id=dict(
                type='str',
                required=True
            ),
            expiry=dict(
                type='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False,
            passwords=None,
        )
        self.resource_group = None
        self.registry_name = None
        self.name = None
        self.expiry = None
        self.token_id = None

        super(AzureRMContainerRegistryTokenPassword, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec):
            setattr(self, key, kwargs[key])

        # Create dict form input, without None value
        password_template = dict(name=self.name,
                                 token_id=self.token_id,
                                 expiry=datetime.strptime(self.expiry, "%Y/%m/%d") if self.expiry else None)
        try:
            response = self.containerregistrytoken_client.registries.begin_generate_credentials(resource_group_name=self.resource_group,
                                                                                                registry_name=self.registry_name,
                                                                                                generate_credentials_parameters=password_template)

        except Exception as ec:
            self.fail("Generate token password occur exception, Exception as {0}".format(str(ec)))

        if isinstance(response, LROPoller):
            response = self.get_poller_result(response)

        self.results = response.as_dict()
        self.results['changed'] = True
        return self.results


def main():
    AzureRMContainerRegistryTokenPassword()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Haiyuan Zhang, <haiyzhan@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_adpassword_info

version_added: "0.2.0"

short_description: Get application password info

description:
        - Get application password info.

options:
    app_id:
        description:
            - The application ID.
        type: str
    service_principal_object_id:
        description:
            - The service principal object ID.
        type: str
    key_id:
        description:
            - The password key ID.
        type: str
    end_date:
        description:
            - Date or datemtime after which credentials expire.
            - Default value is one year after current time.
        type: str
    value:
        description:
            - The application password value.
            - Length greater than 18 characters.
        type: str
    app_object_id:
        description:
            - The application object ID.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    haiyuan_zhang (@haiyuazhang)
    Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: get ad password info
  azure_rm_adpassword_info:
    app_id: "{{ app_id }}"
    key_id: "{{ key_id }}"
'''

RETURN = '''
passwords:
    description:
        - The password info.
    returned: success
    type: dict
    contains:
        custom_key_identifier:
            description:
                - Custom key identifier.
            type: str
            returned: always
            sample: None
        end_date:
            description:
                - Date or datemtime after which credentials expire.
                - Default value is one year after current time.
            type: str
            returned: always
            sample: "2021-06-18T06:51:25.508304+00:00"
        key_id:
            description:
                - The password key ID.
            type: str
            returned: always
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        start_date:
            description:
                - Date or datetime at which credentials become valid.
                - Default value is current time
            type: str
            returned: always
            sample: "2020-06-18T06:51:25.508304+00:00"

'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    import asyncio
    from msgraph.generated.applications.applications_request_builder import ApplicationsRequestBuilder
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADPasswordInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            app_id=dict(type='str'),
            app_object_id=dict(type='str'),
            service_principal_object_id=dict(type='str'),
            key_id=dict(type='str'),
            value=dict(type='str'),
            end_date=dict(type='str'),
        )

        self.app_id = None
        self.service_principal_object_id = None
        self.app_object_id = None
        self.key_id = None
        self.value = None
        self.end_date = None
        self.results = dict(changed=False)

        self.client = None

        super(AzureRMADPasswordInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                    supports_tags=False,
                                                    supports_check_mode=True,
                                                    is_ad_resource=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self._client = self.get_msgraph_client()
        self.resolve_app_obj_id()
        passwords = self.get_all_passwords()

        if self.key_id:
            filtered = [pd for pd in passwords if str(pd.key_id) == self.key_id]
            self.results['passwords'] = [self.to_dict(pd) for pd in filtered]
        else:
            self.results['passwords'] = [self.to_dict(pd) for pd in passwords]

        return self.results

    def resolve_app_obj_id(self):
        try:
            if self.app_object_id is not None:
                return
            elif self.app_id or self.service_principal_object_id:
                if not self.app_id:
                    sp = asyncio.get_event_loop().run_until_complete(self.get_service_principal())
                    self.app_id = sp.app_id
                if not self.app_id:
                    self.fail("can't resolve app via service principal object id {0}".format(
                        self.service_principal_object_id))

                apps = asyncio.get_event_loop().run_until_complete(self.get_applications())
                result = list(apps.value)
                if result:
                    self.app_object_id = result[0].id
                else:
                    self.fail("can't resolve app via app id {0}".format(self.app_id))
            else:
                self.fail("one of the [app_id, app_object_id, service_principal_object_id] must be set")

        except Exception as ge:
            self.fail("error in resolve app_object_id {0}".format(str(ge)))

    def get_all_passwords(self):

        try:
            application = asyncio.get_event_loop().run_until_complete(self.get_application())
            passwordCredentials = application.password_credentials
            return passwordCredentials
        except Exception as ge:
            self.fail("failed to fetch passwords for app {0}: {1}".format(self.app_object_id, str(ge)))

    def to_dict(self, pd):
        return dict(
            end_date=pd.end_date_time,
            start_date=pd.start_date_time,
            key_id=str(pd.key_id),
            custom_key_identifier=str(pd.custom_key_identifier)
        )

    async def get_service_principal(self):
        return await self._client.service_principals.by_service_principal_id(self.service_principal_object_id).get()

    async def get_applications(self):
        request_configuration = ApplicationsRequestBuilder.ApplicationsRequestBuilderGetRequestConfiguration(
            query_parameters=ApplicationsRequestBuilder.ApplicationsRequestBuilderGetQueryParameters(
                filter="appId eq '{0}'".format(self.app_id),
            ),
        )
        return await self._client.applications.get(request_configuration=request_configuration)

    async def get_application(self):
        return await self._client.applications.by_application_id(self.app_object_id).get()


def main():
    AzureRMADPasswordInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_network_info

short_description: Retrieves facts about docker network

description:
  - Retrieves facts about a docker network.
  - Essentially returns the output of C(docker network inspect <name>), similar to what M(community.docker.docker_network)
    returns for a non-absent network.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
  - community.docker._attributes.info_module
  - community.docker._attributes.idempotent_not_modify_state

options:
  name:
    description:
      - The name of the network to inspect.
      - When identifying an existing network name may be a name or a long or short network ID.
    type: str
    required: true

author:
  - "Dave Bendit (@DBendit)"

requirements:
  - "Docker API >= 1.25"
"""

EXAMPLES = r"""
---
- name: Get infos on network
  community.docker.docker_network_info:
    name: mydata
  register: result

- name: Does network exist?
  ansible.builtin.debug:
    msg: "The network {{ 'exists' if result.exists else 'does not exist' }}"

- name: Print information about network
  ansible.builtin.debug:
    var: result.network
  when: result.exists
"""

RETURN = r"""
exists:
  description:
    - Returns whether the network exists.
  type: bool
  returned: always
  sample: true
network:
  description:
    - Facts representing the current state of the network. Matches the docker inspection output.
    - Will be V(none) if network does not exist.
  returned: always
  type: dict
  sample: {
    "Attachable": false,
    "ConfigFrom": {"Network": ""},
    "ConfigOnly": false,
    "Containers": {},
    "Created": "2018-12-07T01:47:51.250835114-06:00",
    "Driver": "bridge",
    "EnableIPv6": false,
    "IPAM": {
      "Config": [
        {
          "Gateway": "192.168.96.1",
          "Subnet": "192.168.96.0/20",
        },
      ],
      "Driver": "default",
      "Options": null,
    },
    "Id": "0856968545f22026c41c2c7c3d448319d3b4a6a03a40b148b3ac4031696d1c0a",
    "Ingress": false,
    "Internal": false,
    "Labels": {},
    "Name": "ansible-test-f2700bba",
    "Options": {},
    "Scope": "local",
  }
"""

import traceback

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        network = client.get_network(client.module.params["name"])

        client.module.exit_json(
            changed=False,
            exists=bool(network),
            network=network,
        )
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()
